"""
Classifies: CHEBI:24828 indoles
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_indoles(smiles: str):
    """
    Determines if a molecule contains an indole skeleton.
    
    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule contains indole skeleton, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # SMARTS patterns for indole skeleton variations
    patterns = [
        'c12ccccc1[nH]cc2',  # Basic indole
        'c12ccccc1n(*)cc2',  # N-substituted indole
        'C1=C2C=CC=CC2=CN1',  # Alternative representation
        'c1ccc2[nH]ccc2c1',   # Another representation
        'C1=CC=C2C(=C1)C=CN2', # Yet another representation
        '[nH]1ccc2ccccc12',    # Reversed representation
        'c1ccc2c(c1)cc[nH]2',  # Another variation
        'C1=CC2=C(C=C1)NC=C2'  # Additional variation
    ]

    for pattern in patterns:
        patt = Chem.MolFromSmarts(pattern)
        if patt and mol.HasSubstructMatch(patt):
            # Check if it's N-substituted
            n_sub_pattern = Chem.MolFromSmarts('c12ccccc1n(*)cc2')
            if n_sub_pattern and mol.HasSubstructMatch(n_sub_pattern):
                return True, "Contains N-substituted indole skeleton"
            return True, "Contains indole skeleton"

    # Check for fused indole systems
    fused_patterns = [
        'c12c3ccccc3[nH]c1cc2',  # Fused indole
        'c1cc2c(cc1)[nH]c1ccccc12',  # Another fused system
        'C1=CC2=C(C=C1)C1=C(N2)C=CC=C1'  # Additional fused system
    ]

    for pattern in fused_patterns:
        patt = Chem.MolFromSmarts(pattern)
        if patt and mol.HasSubstructMatch(patt):
            return True, "Contains indole skeleton"

    # Check for reduced indole (indoline) patterns
    reduced_patterns = [
        'C1CN2c3ccccc3CC2=C1',  # Indoline
        'C1Cc2ccccc2N1',        # Another indoline representation
        'C1=CC2=C(C=C1)NC=C2'   # Additional indoline pattern
    ]

    for pattern in reduced_patterns:
        patt = Chem.MolFromSmarts(pattern)
        if patt and mol.HasSubstructMatch(patt):
            return True, "Contains indole skeleton"

    return False, "No indole skeleton found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24828',
                          'name': 'indoles',
                          'definition': 'Any compound containing an indole '
                                        'skeleton.',
                          'parents': ['CHEBI:22728']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.6958250497017893 is too low.\n'
               "True positives: [('N1C2=C(C=CC(=C2)C)C=C1', 'Contains indole "
               "skeleton'), ('C1=CC=C2C(=C1)C=CC3=C2C=C(N3)C(=O)O', 'Contains "
               "indole skeleton'), "
               "('O=C(NCCC1=CC=C(O)C=C1)CC=2C3=C(C=CC=C3)NC2', 'Contains "
               "indole skeleton'), "
               "('C[C@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@H]1CN(C)C(=O)CCC(F)(F)F)C4=CC=CC=C4N2C)[C@@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('FCCCCCN1C=C(C(=O)NC23CC4CC(C2)CC(C3)C4)C=5C1=CC=CC5', "
               "'Contains N-substituted indole skeleton'), "
               "('CC(=O)NCCC1=C(NC2=C1C=C(C=C2)OC)C3=CC=CC=C3', 'Contains "
               "indole skeleton'), ('O[C@@H](CC=1C=2C(NC1)=CC=CC2)C(O)=O', "
               "'Contains indole skeleton'), "
               "('Cl[C@H]1[C@](C=C)([C@@H]2NC(=O)S[C@@]23C=4C=5C(=CC=CC5C([C@H]3C1)(C)C)NC4)C', "
               "'Contains indole skeleton'), "
               "('NC(=O)c1cc2cc(ccc2o1)N1CCN(CCCCc2c[nH]c3ccc(cc23)C#N)CC1', "
               "'Contains indole skeleton'), "
               "('FC1=CC=2NC=C(CC(N)C(O)=O)C2C=C1', 'Contains indole "
               "skeleton'), ('C=1C2=C(C=C(C(ON3C4=C(N=N3)C=CC=C4)=O)C1)C=CN2', "
               "'Contains indole skeleton'), ('N1C=2C(=C(CN)C=CC2)C=C1', "
               "'Contains indole skeleton'), ('CN(C)CCc1c[nH]c2ccc(O)cc12', "
               "'Contains indole skeleton'), "
               "('C1=CC=C2C(=C1)C(=CN2)CCN=C(N)NC(=O)C3=CC=C(C=C3)Cl', "
               "'Contains indole skeleton'), "
               "('CC(=O)N1C2=CC=CC=C2C(=C1Cl)NC(=O)CCl', 'Contains "
               "N-substituted indole skeleton'), "
               "('N1C(=C(C(=C1C(=O)O)C=2C3=CC=CC=C3NC2)C=4C5=CC=CC=C5NC4)C(O)=O', "
               "'Contains indole skeleton'), "
               "('C[C@@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@H]1CN(C)C(=O)C4=CC=NC=C4)C5=CC=CC=C5N2C)[C@@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('NC1=C(F)C(=NC(C(=O)OCC#N)=C1Cl)C1=C(F)C2=C(C=CN2)C=C1', "
               "'Contains indole skeleton'), "
               "('CN1C=C(C2=CC=CC=C21)C=C3C4=C(C=CC(=C4)S(=O)(=O)N)NC3=O', "
               "'Contains N-substituted indole skeleton'), "
               "('CN1CCC2=CC=CC=C2CC3=C(CC1)C4=CC=CC=C4N3', 'Contains indole "
               "skeleton'), "
               "('O=C1N[C@@H](Cc2c[nH]c3ccccc23)C(=O)N[C@H]1Cc1ccccc1', "
               "'Contains indole skeleton'), ('COC1=CC=CC2=C1C(=CN2)CN=C=S', "
               "'Contains indole skeleton'), "
               "('ClC1=CC=C(C(=O)N2C(=C(C3=C2C=CC(OC)=C3)CC(OCCCCCCC)=O)C)C=C1', "
               "'Contains N-substituted indole skeleton'), "
               "('C[C@@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@@H]1CN(C)C(=O)NC4=CC(=CC=C4)OC)C5=CC=CC=C5N2C)[C@@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('N(CCC=1C=2C(NC1)=CC=CC2)(CC)CC', 'Contains indole "
               "skeleton'), "
               "('Cl.C1[C@@H](C(C2=CN(C3=C2C=CC=C3)C)=O)CC4=C(C1)N=CN4', "
               "'Contains N-substituted indole skeleton'), "
               "('C[C@@H]1CN(C(=O)CC2=C(C=CC(=C2)NC(=O)CC3=CN(C4=CC=CC=C43)C)O[C@H]1CNC)[C@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('O(N1C=2C(C(=C1)C(OC)=O)=CC=CC2)C', 'Contains N-substituted "
               "indole skeleton'), ('OC1=CC=2C(=C(NC2C=C1)C)CC(O)=O', "
               "'Contains indole skeleton'), "
               "('C(\\\\[C@H](CC(C)C)NC([C@H](CC(C)C)NC([C@H](CC=1C2=CC=CC=C2NC1)NC(=O)CN3CCOCC3)=O)=O)=C/S(C)(=O)=O', "
               "'Contains indole skeleton'), "
               "('S1C(=NC=C1)C(=O)C=2C3=C(C=CC=C3)NC2', 'Contains indole "
               "skeleton'), "
               "('O(C1=CC=C(C(C=2C3=C(C=CC=C3)NC2)C=4C5=C(C=CC=C5)NC4)C=C1)CCC', "
               "'Contains indole skeleton'), "
               "('S(C=1C2=C(NC1)C=CC=C2)C3=NCCN3', 'Contains indole "
               "skeleton'), "
               "('C[C@@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@@H]1CN(C)C(=O)NC(C)C)C4=CC=CC=C4N2C)[C@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('CC1=CC2=C(C=C1)C(=CN2)CCN', 'Contains indole skeleton'), "
               "('COC1=CC=CC2=C1C(=C(N2)C(=O)NN)N', 'Contains indole "
               "skeleton'), "
               "('C[C@@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@@H]1CN(C)S(=O)(=O)C4=CC=C(C=C4)OC)C5=CC=CC=C5N2C)[C@@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('C1=2C(=CNC1=CC=CC2)CC#N', 'Contains indole skeleton'), "
               "('O=C(OCCC=1NC=2C=CC=CC2C1)CC=3C4=C(C=CC=C4)NC3', 'Contains "
               "indole skeleton'), ('O(C(CC=1C=2C(NC1)=CC=CC2)C(O)=O)C', "
               "'Contains indole skeleton'), "
               "('[H][C@@]12CCCN1C(=O)[C@H](Cc1c[nH]c3ccccc13)NC2=O', "
               "'Contains indole skeleton'), "
               "('C(CC1CCNCC1)C=2C=3C(NC2)=CC=CC3', 'Contains indole "
               "skeleton'), ('N1C=C(C2=C1C=CC(=C2Cl)Br)O', 'Contains indole "
               "skeleton'), "
               "('C[C@@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@H]1CN(C)C(=O)NC(C)C)C4=CC=CC=C4N2C)[C@@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('O=C1[C@@]23C(=O)NC([C@@H]2[C@H](C)[C@@]4([C@H]([C@@H]3C=CC[C@@H](CC([C@H]5[C@H]1O5)=O)C)O4)C)CC=6C7=C(C=CC=C7)NC6', "
               "'Contains indole skeleton'), "
               "('C[C@@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@H]1CN(C)S(=O)(=O)C4=CSC=C4)C5=CC=CC=C5N2C)[C@@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('C=1(NC(=NC1)C(=O)C=2C=3C=CC(=CC3NC2)O)C=4C=5C=CC(=CC5NC4)Br', "
               "'Contains indole skeleton'), "
               "('C1=CC=C2C(=C1)C(=CN2)C(=O)C(=O)NC3=NC=CS3', 'Contains indole "
               "skeleton'), ('C1=C2C(=CNC2=CC=C1C[C@@H]3NC(=O)OC3)CCN(C)C', "
               "'Contains indole skeleton'), "
               "('C12=C(NC(=C1)C)C=CC=C2OC[C@@H](OC(C3=CC=CC=C3)=O)CNC(C)(C)C', "
               "'Contains indole skeleton'), "
               "('O1C2(C1C3C4(C(C2C)C(NC4=O)C(C=5C=6C(NC5)=CC=CC6)C)C(=O)CCC(=O)C(=O)C(=CC(CC=C3)C)C)C', "
               "'Contains indole skeleton'), "
               "('C[C@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@H]1CN(C)C(=O)NC4=C(C=C(C=C4)OC)OC)C5=CC=CC=C5N2C)[C@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('O=C(C(O)CC=1C2=C(C=CC=C2)NC1)C(CC)C', 'Contains indole "
               "skeleton'), "
               "('C[C@@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@@H]1CN(C)S(=O)(=O)CC4=CC=CC=C4)C5=CC=CC=C5N2C)[C@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('OC1=C(C2=C(N1)C=CC(=C2)C#N)C1=NC=C(CN2CCOCC2)C=C1', "
               "'Contains indole skeleton'), "
               "('C1=C2NC=C(C2=CC=C1)CCNC(CC[NH3+])=O', 'Contains indole "
               "skeleton'), ('CCOC(=O)C1=C(C2=C(N1)C=CC(=C2)OC)C', 'Contains "
               "indole skeleton'), "
               "('FC=1C=C(C(=O)N2CC(C3=C(NC=4C3=CC=CC4)C(=C2)C(OC(C)C)=O)(C)C)C=CC1F', "
               "'Contains indole skeleton'), "
               "('C[C@@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@@H]1CN(C)C(=O)N[C@H](C)C(C)C)C4=CC=CC=C4N2C)[C@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('O=C(C1C(C1(C)C)(C)C)C=2C=3C(NC2)=CC=CC3', 'Contains indole "
               "skeleton'), "
               "('C[C@H]1C\\\\C=C\\\\[C@H]2[C@H](O)C(C)=C(C)[C@H]3[C@H](Cc4c[nH]c5ccccc45)NC(=O)[C@@]23C(=O)CCC(=O)C(=O)\\\\C(C)=C\\\\1', "
               "'Contains indole skeleton'), "
               "('C[C@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@@H]1CN(C)C(=O)NC4=CC(=CC=C4)F)C5=CC=CC=C5N2C)[C@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('CNS(=O)(=O)Cc1ccc2[nH]cc(CCN(C)C)c2c1', 'Contains indole "
               "skeleton'), ('O=C(O)C=1C2=C(C=CC(=C2)CC=C(C)C)NC1', 'Contains "
               "indole skeleton'), "
               "('C[C@@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@@H]1CN(C)S(=O)(=O)C4=CC=C(C=C4)Cl)C5=CC=CC=C5N2C)[C@@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('C1=2C(NC=C1CCN(C)C)=CC=C(C2)CN3N=CN=C3.C=1(C=CC=CC1)C(O)=O', "
               "'Contains indole skeleton'), "
               "('ClC1=CC=2C([P@@](OC)(=O)C3=CC(=CC(=C3)/C=C/C#N)C)=C(NC2C=C1)C(=O)N', "
               "'Contains indole skeleton'), "
               "('C=1C=C2C(=CC1)N(C=C2C[C@@H](C(CC(C)C)=O)O)C', 'Contains "
               "N-substituted indole skeleton'), "
               "('COC(=O)C1=C(\\\\N=N\\\\N2CC3C[C@H](C2)C2=CC=CC(=O)N2C3)C2=C(N1)C=CC=C2', "
               "'Contains indole skeleton'), "
               "('OC(=O)CNC(=O)CC1=CNC2=CC=C(O)C=C12', 'Contains indole "
               "skeleton'), "
               "('C[C@@H]1CN(C(=O)C2=C(C=CC(=C2)NC(=O)CC3=CN(C4=CC=CC=C43)C)O[C@@H]1CN(C)CC5=CC6=C(C=C5)OCO6)[C@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('C1=CC=CC=2C(=CNC12)CC=[N+](O)[O-]', 'Contains indole "
               "skeleton'), "
               "('C12=CC=CC=C1C(=CN2)[C@@H](C)[C@]3(C(N=C(O3)N)=O)[H]', "
               "'Contains indole skeleton'), "
               "('O(C1=CC=2C(CC(N)C(O)=O)=CNC2C=C1)C', 'Contains indole "
               "skeleton'), "
               "('C[C@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@H]1CN(C)C(=O)N[C@@H](C)C(C)C)C4=CC=CC=C4N2C)[C@@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('CC1=C(C2=CC=CC=C2N1C)P(=O)(C3=CC=CC=C3)C4=CC=CC=C4', "
               "'Contains N-substituted indole skeleton'), "
               "('C[C@@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@@H]1CN(C)S(=O)(=O)C4=CC=CC(=C4)C)C5=CC=CC=C5N2C)[C@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('O=C(OCCC=1C2=C(C=CC=C2)NC1)CC=3C4=C(C=CC=C4)NC3', 'Contains "
               "indole skeleton'), "
               "('O=C(N[C@H](C(=O)NCCC=1C2=C(C=CC=C2)NC1)C(C)C)C(=O)C(C)C', "
               "'Contains indole skeleton'), "
               "('CN1C=C(C2=CC=CC=C21)C=NNC(=O)CCCCCCC(=O)NC3=CC=CC=C3O', "
               "'Contains N-substituted indole skeleton'), "
               "('CC(C)COC(=O)N[C@@H](CC1=CNC2=CC=CC=C21)C(=O)NNC=C3C=C(C(=O)C(=C3)OC)[N+](=O)[O-]', "
               "'Contains indole skeleton'), "
               "('O=C(O)CC=1C2=C(C=CC=C2)N(C1)C3OC(C(O)C(C3O)O)CO', 'Contains "
               "N-substituted indole skeleton'), "
               "('C[C@@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@H]1CN(C)S(=O)(=O)C4=CC=CC=C4OC)C5=CC=CC=C5N2C)[C@@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('C12=C3C(=C(C(=C1NC(=C2)C(SC)=O)OC)O)N(CC3)C(=O)C4=CC5=C(N4)C(=CC6=C5C(=CN6C(=O)C7=CC8=C(N7)C=C(C(=C8)O)OC)C)O', "
               "'Contains indole skeleton'), "
               "('C[C@@H]1O[C@@H](OCCCCCCCCCCC(O)=O)[C@H](O)C[C@H]1OC(=O)c1c[nH]c2ccccc12', "
               "'Contains indole skeleton'), "
               "('Cn1ccc2cc(NC(=O)Nc3cccnc3)ccc12', 'Contains N-substituted "
               "indole skeleton'), "
               "('C12=CC=CC=C1NC(=C2)C(=O)N[C@@H](CC=3C=CC=C(C3)F)C(N[C@H](C(=O)[H])C[C@H]4C(NCC4)=O)=O', "
               "'Contains indole skeleton'), "
               "('C[C@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@@H]1CN(C)S(=O)(=O)C4=C(ON=C4C)C)C5=CC=CC=C5N2C)[C@@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('O=C([C@@H](O)CC=1C2=C(C=CC=C2)NC1)CC3=CC=C(O)C=C3', "
               "'Contains indole skeleton'), "
               "('CCOC(=O)C1=C(N(C2=CC(=C(C=C21)O)Br)C)CNC3=CC=NC=C3', "
               "'Contains N-substituted indole skeleton'), "
               "('O[C@@H]1[C@@]2([C@]([C@H](C)CC1)(CCC([C@H]2CC=3C4=C(C=CC=C4)NC3)=C)C)CCC=C(C)C', "
               "'Contains indole skeleton'), "
               "('C=1(C=2NC=C3C[C@@H](CO)NC([C@H](C(C)C)N(C)C(C32)=CC1)=O)[C@](CCC=C(C)C)(C=C)C', "
               "'Contains indole skeleton'), "
               "('O=C(NCCC=1C=2C(NC1)=CC=CC2)CCCCCCCCCCCCCCCCCCCCCCC', "
               "'Contains indole skeleton'), "
               "('C[C@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@@H]1CN(C)C)C4=CC=CC=C4N2C)[C@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('C[C@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@H]1CN(C)C(=O)OCC(C)C)C4=CC=CC=C4N2C)[C@@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('C[C@@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@H]1CN(C)S(=O)(=O)C4=CC=C(C=C4)Cl)C5=CC=CC=C5N2C)[C@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('C[C@@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@@H]1CN(C)C(=O)OCC(C)C)C4=CC=CC=C4N2C)[C@@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('O=C1C(O)=C(C(=O)C(=C1C=2C3=C(C=C(/C=C/C(=C)C)C=C3)NC2)O)C=4C5=C(C=C(/C=C/C(=C)C)C=C5)NC4', "
               "'Contains indole skeleton'), ('O(C1=CC2=C(NC=C2C=O)C=C1)C', "
               "'Contains indole skeleton'), "
               "('BrC1=CC=2NC=C(C2C=C1)CCNC(=O)CC', 'Contains indole "
               "skeleton')]\n"
               'False positives: '
               "[('CCC(C)C1=CC=CC=C1NC(=O)C2=CC3=C(N2)C(=CC=C3)[N+](=O)[O-]', "
               "'Contains indole skeleton'), "
               "('CC(O)C(O)=O.Cc1[nH]c2ccccc2c1CCNCc1ccc(\\\\C=C\\\\C(=O)NO)cc1', "
               "'Contains indole skeleton'), "
               "('O=C(N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC=3NC=NC3)CC(O)=O', "
               "'Contains indole skeleton'), "
               "('O1[C@@H]([C@@H](O)[C@H](O)[C@@H](O)[C@@H]1OC=2C3=C(NC=C3CC#N)C=CC2)CO[C@@H]4O[C@@H]([C@@H](O)[C@H](O)[C@H]4O)CO', "
               "'Contains indole skeleton'), "
               "('O(C(C)(C)C)C(=O)C1=CC=C(CN2CCN(CC2)C=3C4=C(NC=C4)C=CC3)C=C1', "
               "'Contains indole skeleton'), "
               "('CCN1C2=CC=CC=C2C3=C1N=C(N=N3)SCC(=O)C4=CC=CC=C4', 'Contains "
               "N-substituted indole skeleton'), "
               "('CO[C@H]1[C@@H](C[C@@H]2CN3CCc4c([nH]c5ccccc45)[C@H]3C[C@@H]2[C@@H]1C(=O)OC)OC(=O)c1cc(OC)c(OC)c(OC)c1', "
               "'Contains indole skeleton'), "
               "('O=C(NCC(=O)N[C@@H](CC1=CC=CC=C1)C(O)=O)[C@@H](N)CC=2C=3C(NC2)=CC=CC3', "
               "'Contains indole skeleton'), "
               "('O[C@]12C=3C4=C5C=CC=C4NC3C(C)(C)C([C@]1([N+]#[C-])[C@@](C=C)(CC[C@@H]2C5(C)C)C)=C', "
               "'Contains indole skeleton'), "
               "('C1CC(C2=C(C1)C3=C(N2)C=CC(=C3)Cl)C(=O)N4CCN(CC4)C(=O)C5=CC=CO5', "
               "'Contains indole skeleton'), "
               "('O=C1N[C@H](C(=O)N[C@H](C(=O)O)CCCNC(=N)N)CC(=O)NCCCC[C@@H]2NC(=O)[C@@H](NC([C@H]([C@H](OC(C[C@H]3C(N[C@H](C(N[C@H]1CCC(=O)OC[C@H](NC(=O)[C@H]4C(C(=O)[C@@H](NC2=O)CC5=CC=CC=C5)CCC4)C(=O)N3)=O)CC(C)C)=O)=O)C)NC(=O)[C@@H](NC(=O)[C@H]6N(C(=O)[C@@H](N)C(C)C)CCC6)CC=7C8=C(C=CC=C8)NC7)=O)CC9=CC=C(O)C=C9', "
               "'Contains indole skeleton'), "
               "('O=C(N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(O)=O)CC=3NC=NC3', "
               "'Contains indole skeleton'), "
               "('CN1C2=C(C=CC(=C2)OC)C3=C1[C@H](NCC34CCN(CC4)C(=O)NC5=CC6=C(C=C5)OCO6)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('CSCC[C@H](NC(=O)[C@@H](N)CC(N)=O)C(=O)N[C@@H](Cc1c[nH]c2ccccc12)C(=O)N[C@@H](CC(N)=O)C(O)=O', "
               "'Contains indole skeleton'), "
               "('O=C/1N[C@@](O)(C(=O)N\\\\C1=C/C=2C3=C(C=CC=C3)NC2C(C=C)(C)C)C', "
               "'Contains indole skeleton'), "
               "('CN1C2=C(C=CC(=C2)OC)C3=C1[C@H](N(CC34CCN(CC4)C(=O)C5CCCC5)S(=O)(=O)C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('O=C(N[C@@H](CCC(=O)N)C(=O)N[C@@H]([C@H](O)C)C(O)=O)[C@@H](N)CC=1C=2C(NC1)=CC=CC2', "
               "'Contains indole skeleton'), "
               "('O=C(N[C@@H](C(C)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCCCN)CC=1C=2C(NC1)=CC=CC2', "
               "'Contains indole skeleton'), "
               "('O=C1O[C@@H]([C@H](NC(=O)[C@@H](NC(=O)[C@H](NC(=O)[C@@H](NC(=O)CCCCCCCCC)CC=2C3=C(C=CC=C3)NC2)CCC(=O)O)[C@@H](O)C(=O)N)C(=O)N(CC(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](CCCCN)C(N[C@H](C(NCC(N[C@@H](C(N[C@H](C(N[C@H]1[C@H](CC)C)=O)CCC(=O)O)=O)CC(=O)N)=O)=O)[C@@H](OC)C(=O)O)=O)CC(=O)O)C)C)C', "
               "'Contains indole skeleton'), "
               "('C(=C/C(SCCNC(CCNC(=O)[C@@H](C(COP(OP(OC[C@H]1O[C@@H](N2C3=C(C(=NC=N3)N)N=C2)[C@@H]([C@@H]1OP([O-])([O-])=O)O)(=O)[O-])(=O)[O-])(C)C)O)=O)=O)\\\\CCCC[C@H](O[C@@H]4O[C@@H](C)[C@@H](C[C@H]4O)OC(C=5C=6C=CC=CC6NC5)=O)C', "
               "'Contains indole skeleton'), "
               "('O=C(N1C(C(O)=NC(C(O)=NC(C(O)=NC(C(O)=NC(C(O)=NC(C(O)=NC(CO)CC=2C3=C(C=CC=C3)NC2)CCC(O)=N)CCC(O)=N)C(CC)C)(C)C)C(C)C)CCC1)C(N=C(O)C(N=C(O)C(N=C(O)C(N=C(O)C(N=C(O)C(N=C(O)C(N=C(O)C(N=C(O)C(N=C(O)C(N=C(O)C(N=C(O)C(N=C(O)C)(C)C)C)C)(C)C)(C)C)CCC(O)=N)(C)C)(C)C)(C)C)CO)CC(C)C)(C)C', "
               "'Contains indole skeleton'), "
               "('O=C(N[C@@H](CC=1NC=NC1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(=O)N)CC=2C=3C(NC2)=CC=CC3', "
               "'Contains indole skeleton'), "
               "('OC[C@H](O)Cc1nccc2c3ccccc3[nH]c12', 'Contains indole "
               "skeleton'), ('ON(O)[C@@H](Cc1c[nH]c2ccccc12)C(O)=O', 'Contains "
               "indole skeleton'), "
               "('C=C[C@]1(CN2CCC=3C=4C=CC(=C([C@]5(CCCN5C)[H])C4NC3[C@@]2(C[C@@]1(C[C@]6(C7=C(CCN6C)C8=CC=CC=C8N7)[H])[H])[H])O)[H]', "
               "'Contains indole skeleton'), "
               "('O=C(N[C@H](C(=O)N[C@@H](CC(O)=O)C(O)=O)CO)[C@@H](N)CC=1C=2C(NC1)=CC=CC2', "
               "'Contains indole skeleton'), "
               "('CC(C(NC(CC=1C2=C(C=C(C=C2)O)NC1)=O)C(O)=O)C', 'Contains "
               "indole skeleton'), "
               "('OCC1C2N3C(CC1C(C3)=CC)C=4NC5=C(C4C2)C=CC(OC)=C5', 'Contains "
               "indole skeleton'), "
               "('COC(=O)[C@@]1(O)C[C@H]2O[C@]1(C)n1c3ccccc3c3c4CNC(=O)c4c4c5ccccc5n2c4c13', "
               "'Contains N-substituted indole skeleton'), "
               "('O=C(N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(=O)N[C@@H](CCCN=C(N)N)C(O)=O)[C@H]3NCCC3', "
               "'Contains indole skeleton'), "
               "('O=C1N2[C@H](C(=O)N3[C@@]1(O)C(=O)C=4C5=C(C=C(OC)C=C5)N(C4[C@@H]3C=C(C)C)CC=C(C)C)CCC2', "
               "'Contains N-substituted indole skeleton'), "
               "('CN1C2=C(C=CC(=C2)OC)C3=C1[C@H](N(CC34CCN(CC4)CC5=CC=CC=N5)C(=O)NC6=CC=C(C=C6)F)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('O(C=1C=2NC3=C(C2C=C(C1)C=O)C=C(OC)C=C3)C', 'Contains indole "
               "skeleton'), "
               "('O=C1N2[C@@H](C3=C4C(NC=C4C[C@H]2C(=C1C(=O)C)O)=CC=C3)C=C(C)C', "
               "'Contains indole skeleton'), "
               "('O=C(NC=CC1=CC=C(O[C@@H]2O[C@@H]([C@H](NC(=O)C)[C@@H]([C@H]2O)O)C)C=C1)[C@@H](N[C@@H](C(=O)NCCC3=CC=CC=C3)CC4=CC=CC=C4)CC=5C6=C(C=CC=C6)N(C5)C', "
               "'Contains N-substituted indole skeleton'), "
               "('[H]C1C[C@H](C)OC(=O)C[C@@H](NC(=O)[C@@H](Cc2c(Br)[nH]c3ccccc23)N(C)C(=O)[C@H](C)NC(=O)[C@@H](C)C\\\\C(C)=C\\\\1)c1ccc(O)cc1', "
               "'Contains indole skeleton'), "
               "('S([C@]1(N(C(=O)[C@@](OC)(CC=2C3=C(C=CC=C3)NC2)NC1=O)C)CO)C', "
               "'Contains indole skeleton'), "
               "('ClC1=CC=2NC(Cl)=C(C2C=C1)C[C@H]3NC(=O)[C@H](NC(=O)[C@H](NC(=O)[C@H](CO)NC([C@H](NC([C@@H](NC3=O)[C@@H](CC)C)=O)CCC(=O)O)=O)C(C)C)CC=4C5=C(C=CC=C5)NC4', "
               "'Contains indole skeleton'), "
               "('O=C1N2[C@H](C(=O)N3[C@]1(O)[C@@H](OC)C=4C5=C(C=C(OC)C=C5)NC4[C@@H]3C=C(C)C)CCC2', "
               "'Contains indole skeleton'), "
               "('CN1C2=C(C=CC(=C2)OC)C3=C1[C@@H](N(CC34CCN(CC4)CC5CC5)CC6=CC=CC=C6OC)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('C1=C(C(=CC=2NC=CC12)O[C@@H]3[C@H]([C@@H]([C@H]([C@@H](O3)C(O)=O)O)O)O)O', "
               "'Contains indole skeleton'), "
               "('O[C@]12[C@@]([C@@]3([C@@](CC1)(CC4=C3NC=5C4=C(C(C)(C)C=C)C=CC5)[H])C)(CC[C@]67O[C@](C(O6)(C)C)(C(=O)C=C27)[H])C', "
               "'Contains indole skeleton'), "
               "('O[C@H](Cc1c[nH]c2ccc(OCc3ccccc3)cc12)C(O)=O', 'Contains "
               "indole skeleton'), "
               "('S(CC[C@H](NC(=O)[C@@H](N)CC=1C=2C(NC1)=CC=CC2)C(=O)N[C@@H](CCC(=O)N)C(O)=O)C', "
               "'Contains indole skeleton'), "
               "('O=C([O-])[C@@H](NC(CC1=CNC2=CC=CC=C12)=O)CC(=O)[O-]', "
               "'Contains indole skeleton'), "
               "('CC1=CC=C(C=C1)NC(=O)CSC2=NC3=C(C(=O)N2C)NC4=CC=CC=C43', "
               "'Contains indole skeleton'), "
               "('[H]\\\\C(C)=C1/C[NH2+][C@@]2([H])CC3=C(NC4=CC=CC=C34)C(=O)C[C@]1([H])[C@]2([H])C(=O)OC', "
               "'Contains indole skeleton'), "
               "('O=C(N[C@@H](C(C)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(C)C)CC=1C=2C(NC1)=CC=CC2', "
               "'Contains indole skeleton'), "
               "('S(CC[C@H](NC(=O)[C@@H](NC(=O)[C@@H](N)[C@H](CC)C)CC=1C=2C(NC1)=CC=CC2)C(O)=O)C', "
               "'Contains indole skeleton'), "
               "('COCC(=O)N1CC2(C1)CN([C@@H](C3=C2C4=C(N3)C=C(C=C4)OC)CO)CC5=CC=CC=C5F', "
               "'Contains indole skeleton'), "
               "('O=C(NCC(=O)N[C@@H](CC1=CC=C(O)C=C1)C(O)=O)[C@@H](N)CC=2C=3C(NC2)=CC=CC3', "
               "'Contains indole skeleton'), "
               "('CC1=C(C2=C(N1)C=CC(=C2)F)CCN(CC3=CC=NC=C3)C(=S)NC4=CC=C(C=C4)F', "
               "'Contains indole skeleton'), "
               "('CCC1CCCCN1C(=O)CSC2=NC3=C(C(=O)N2C)NC4=CC=CC=C43', 'Contains "
               "indole skeleton'), "
               "('CN1C2=C(C=CC(=C2)OC)C3=C1[C@@H](N(CC34CCN(CC4)CC5=CN=CC=C5)C(=O)COC)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('O=C1N[C@H](C(=O)N[C@@H]1C(C)C)CC=2C3=C(C(=CC(=C3)CC=C(C)C)CC=C(C)C)NC2C(C=C)(C)C', "
               "'Contains indole skeleton'), "
               "('O=C1N(C(C(=O)NC(C(=O)NCCCCC(C(NC(C(NC1CC=2C3=C(C=CC=C3)NC2)=O)C(CC)C)=O)NC(=O)NC(C(=O)O)C(C)C)CC4=CC=CC=C4)C)C', "
               "'Contains indole skeleton'), "
               "('CC[C@H](C)[C@@H](C(=O)N[C@@]1(CCCC=CCCC[C@](NC(=O)[C@@H](NC(=O)[C@@H](NC(=O)[C@@H](NC1=O)CC(C)C)CS)CCCNC(=N)N)(C)C(=O)N[C@@H](CC2=CNC=N2)C(=O)N[C@@H](CC3=CNC=N3)C(=O)N[C@@H](CO)C(=O)N[C@@H]([C@@H](C)O)C(=O)N)C)NC(=O)[C@H](CCCNC(=N)N)NC(=O)[C@H](CCCNC(=N)N)NC(=O)[C@H](CCCNC(=N)N)NC(=O)[C@H](CC(C)C)NC(=O)[C@H](CCCNC(=N)N)NC(=O)[C@H](CCC(=O)O)NC(=O)[C@H](CC4=CNC5=CC=CC=C54)NC(=O)CCNC(=O)C', "
               "'Contains indole skeleton'), "
               "('C[C@H]1CN(C(=O)C2=C(C3=CC=CC=C3CO[C@@H]1CN(C)C(=O)[C@H](CC4=CC=CC=C4)N(C)C)C5=CC=CC=C5N2C)[C@H](C)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('O=C(N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCCN=C(N)N)CCCCN', "
               "'Contains indole skeleton'), "
               "('[O-]C(=O)C(F)(F)F.[O-]C(=O)C(F)(F)F.NC(=[NH2+])NCCCc1[nH+]ccc2c3ccc(Br)cc3[nH]c12', "
               "'Contains indole skeleton'), "
               "('O=C(N[C@H](C(=O)O)CC=1C2=C(C=CC=C2)NC1)C3N(C(=O)[C@@H](NC(=O)CC(C)C)CCCCNC(=N)N)C\\\\C(\\\\[C@H]3C)=C\\\\C', "
               "'Contains indole skeleton'), "
               "('O=C(N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(O)=O)[C@@H](NC(=O)[C@@H](N)C(C)C)CCC(=O)N', "
               "'Contains indole skeleton'), "
               "('CC(C)C[C@@H]1NC(=O)[C@H](NC(=O)[C@@H]2CCCN2C(=O)[C@@H](CC(O)=O)NC(=O)[C@@H](Cc2c[nH]c3ccccc23)NC1=O)c1cccs1', "
               "'Contains indole skeleton'), "
               "('O=C1N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)NCC(=O)N[C@H](C(=O)N[C@H](C(=O)O)CC2=CC=CC=C2)CC(C)C)CC(=O)N)CCCCN)CO)CC(=O)NCC(=O)N[C@H](C(=O)NCC(N[C@H](C(N[C@H](C(N[C@H]1CC=3C4=C(C=CC=C4)NC3)=O)C)=O)CC(=O)N)=O)CC5=CC=C(O)C=C5', "
               "'Contains indole skeleton'), "
               "('C1CCC2=C(C1)C3=C4N2C(=O)C(=C(C4=CC=C3)O)CC5=CC=CC=C5', "
               "'Contains N-substituted indole skeleton'), "
               "('O=C1C=CC2(C3(C=4NC=5C=C(O[C@@H]6O[C@@H]([C@@H](OC)[C@@H]([C@H]6O)O)CO)C=CC5C4CC3CCC2C1(C)C)C)C', "
               "'Contains indole skeleton'), "
               "('O=C(N1[C@H](CCC1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC=2C=3C(NC2)=CC=CC3)CO', "
               "'Contains indole skeleton'), "
               "('CN1C2=C(C=CC(=C2)OC)C3=C1[C@H](NCC34CCN(CC4)CC5=CC=NC=C5)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('O=C(N1[C@@H](CCC1)C(=O)N[C@@H](CC=2C=3C(NC2)=CC=CC3)C(=O)N[C@@H]([C@H](O)C)C(O)=O)[C@@H](NC(=O)[C@@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CC(C)C)C(C)C)C(C)C)CC4=CC=C(O)C=C4', "
               "'Contains indole skeleton'), "
               "('CN1C2=C(C=CC(=C2)OC)C3=C1[C@@H](N(CC34CCN(CC4)CC5CC5)C(=O)C6=CC=NC=C6)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('C1=CC=C2C(=C1)C3=CC=CC=C3N2CC(COC4=CC=CC5=C4N=CC=C5)O', "
               "'Contains N-substituted indole skeleton'), "
               "('O=C(N1[C@H](CCC1)C(O)=O)[C@@H](NC(=O)[C@H]2NCCC2)CC=3C=4C(NC3)=CC=CC4', "
               "'Contains indole skeleton'), "
               "('CC(=O)N1CCC2(CC1)CN([C@H](C3=C2C4=C(N3)C=C(C=C4)OC)CO)C(=O)C', "
               "'Contains indole skeleton'), "
               "('O=C(N[C@H](C(=O)N[C@@H](CCC(=O)N)C(O)=O)CO)[C@@H](N)CC=1C=2C(NC1)=CC=CC2', "
               "'Contains indole skeleton'), "
               "('CN1CC2(CCN(CC2)S(=O)(=O)C3=CC=CC(=C3)F)C4=C([C@H]1CO)N(C5=C4C=CC(=C5)OC)C', "
               "'Contains N-substituted indole skeleton'), "
               "('S(CC[C@@H]1NC(=O)[C@@H](NC(=O)[C@@H](NC(=O)CNC(=O)[C@@H](NC(=O)C)CC2=CC=C(O)C=C2)CC(=O)N)[C@H](OC(=O)C[C@@H]3NC(=O)[C@H](CO)NC([C@H]4N(C([C@@H](NC([C@@H](NC1=O)CCCCNC(=O)CC[C@H](NC(=O)[C@H](NC([C@@H](NC3=O)CC=5C6=C(C=CC=C6)NC5)=O)CCC(=O)OC)C(=O)N[C@H](C(=O)O)CC7=CC=C(O)C=C7)=O)CC8=CC=C(O)C=C8)=O)CCC4)=O)C)C', "
               "'Contains indole skeleton'), "
               "('C\\\\C=C1\\\\CN2[C@H]3Cc4c([nH]c5ccccc45)[C@@H]2C[C@@H]1[C@H]3C=O', "
               "'Contains indole skeleton'), "
               "('O=C(N[C@@H](CC1=CC=CC=C1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCCCN)CC=2C=3C(NC2)=CC=CC3', "
               "'Contains indole skeleton'), "
               "('O=C1NCC=2C1=C3C(NC4=C3C=CC=C4)=C5N(NC)C6=C(C25)C=CC=C6', "
               "'Contains indole skeleton'), "
               "('O=C(N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCC(O)=O)CCC(=O)N', "
               "'Contains indole skeleton'), "
               "('C1(=CC(=CC=C1)C#N)S(=O)(=O)[N-]C2=CC=C(C=3C(=CNC23)C#N)C', "
               "'Contains indole skeleton'), "
               "('O=C1N([C@H](C(=O)N[C@@H](C(=O)N[C@@H](C)CC(O[C@@H]([C@@H](C(N[C@H]1[C@H](CC)C)=O)NC(=O)C(=NNC2=C(C(=O)OCCCCCCCCCC(C)C)C=C(O)C=C2)C)C3=CC=CC=C3)=O)CC=4C5=C(C(OC)=CC=C5)NC4)[C@@H](O)C6=CC=CC=C6)C', "
               "'Contains indole skeleton'), "
               "('O=C1N[C@H](C(=O)NC2=C1C=CC=C2)CC=3C4=C(C=CC=C4)N(C3)CC=C(C)C', "
               "'Contains N-substituted indole skeleton'), "
               "('CN[C@@H]1Cc2c[nH]c3cccc([C@H]1\\\\C=C(/C)C=O)c23', 'Contains "
               "indole skeleton'), "
               "('CN1C2=C(C=CC(=C2)OC)C3=C1[C@H](N(CC34CCN(CC4)C(=O)C5=CC6=C(C=C5)OCO6)C(=O)NC7=CC=C(C=C7)F)CO', "
               "'Contains N-substituted indole skeleton'), "
               "('O=C(NC(CCCCN)C(O)=O)C(NC(=O)C(N)CC=1C=2C(NC1)=CC=CC2)CCCCN', "
               "'Contains indole skeleton'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC(C)([C@H](C(NCCC(NCCSC([C@@H](CC=4C5=CC=CC=C5NC4)O)=O)=O)=O)O)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'Contains indole skeleton'), "
               "('NC(Cc1c[nH]c2ccc(O)cc12)C(O)=O', 'Contains indole "
               "skeleton'), "
               "('O=C(NC=CC1=CC=C(O[C@@H]2O[C@@H]([C@H](NC(=O)C)[C@@H]([C@H]2O)O)C)C=C1)[C@@H](N[C@@H](C(=O)NCCC3=CC=CC=C3)CC(C)C)CC=4C5=C(C=CC=C5)N(C4)C', "
               "'Contains N-substituted indole skeleton'), "
               "('O(C=1C=2NC=3C(C2C=C(C1)C(OC)=O)=CC=CC3)C', 'Contains indole "
               "skeleton'), ('O1C(C=CC=2C=3NC=4C(C3C=C(C12)C=O)=CC=CC4)(C)C', "
               "'Contains indole skeleton'), "
               "('O=C1[C@H](N[C@H](C(=O)O)CC2=CC=C(O)C=C2)CCC(O)=NCCCC[C@@H]3N=C(O)[C@@H](N=C([C@H]([C@H](OC(C[C@H]4C(=N[C@H](C(=N[C@H]1CCC(=O)OC[C@H](N=C(O)[C@H]5N(C(=O)[C@@H](N=C3O)CC6=CC=C(O)C=C6)CCC5)C(O)=N4)O)CC=7C8=C(C=CC=C8)NC7)O)=O)C)N=C(O)[C@@H](N=C(O)[C@@H](N=C(O)C)CC9=CC=C(O)C=C9)CO)O)CC(C)C', "
               "'Contains indole skeleton'), "
               "('COC(=O)\\\\C=C/NC(=O)c1cc2c3cccc(O)c3[nH]c2c(n1)C(C)=O', "
               "'Contains indole skeleton'), "
               "('S=C1NCCC1C2NC(CC3=C2NC=4C3=CC=CC4)C(O)=O', 'Contains indole "
               "skeleton'), "
               "('O=C(NC(CC=1C=2C(NC1)=CC=CC2)C(O)=O)CCCCCCC/C=C/C\\\\C=C\\\\CCCCC', "
               "'Contains indole skeleton'), "
               "('S1S[C@@]23N([C@H]4N5C=6C=CC=CC6[C@]4(C2)N7C8=C(C=CC=C8)C(=C7)C[C@@]9(SC)N(C)C([C@](COC5)(SC)N(C)C9=O)=O)C([C@]1(N(C)C3=O)CO)=O', "
               "'Contains N-substituted indole skeleton'), "
               "('[H][C@@]12CC3=CNC4=CC=CC(=C34)C1=C[C@H](CN2C(C)(C)C)C(=O)N(CC)CC', "
               "'Contains indole skeleton'), "
               "('CCS(=O)(=O)N1CC2(C1)CN([C@H](C3=C2C4=C(N3)C=C(C=C4)OC)CO)C(=O)CC5=CC=CC=N5', "
               "'Contains indole skeleton'), "
               "('C12=CC=CC=C1NC3=C2[C@]4([C@H]([C@](CC[C@@]4(C3(C)C)[H])(C)C=C)[N+]#[C-])[H]', "
               "'Contains indole skeleton'), "
               "('[H][C@@]12Cc3c[nH]c4cccc(c34)[C@@]1([H])[C@@H](OC(C)=O)[C@@H](C)C[NH+]2C', "
               "'Contains indole skeleton')]\n"
               'False negatives: '
               "[('S1S[C@]23N([C@@H]4[C@H]5O[C@H]5[C@H](C=C4C2)OC(=O)C)C([C@]16N([C@@H]7[C@H]8O[C@H]8[C@H](C=C7C6)OC(=O)CCCCC)C3=O)=O', "
               "'No indole skeleton found'), "
               "('ClC1=C[C@H]2C(=O)[C@]([C@@]1(C=C)C)(N=C=S)C3=C4C(=CC=C3)NC([C@@H]4C2(C)C)=O', "
               "'No indole skeleton found'), "
               "('C1CCC(CC1)NC(=O)N2CCC3=CC=CC=C32', 'No indole skeleton "
               "found'), ('S(C(SC)=N\\\\C=C/1\\\\C=2C(N(OC)C1=O)=CC=CC2)C', "
               "'No indole skeleton found'), "
               "('S(=O)(=O)(C1=CC=C2C(=C1)C(C(N2CC3=CC=CC=C3)=O)=O)N4CC(CC(C4)C)C', "
               "'No indole skeleton found'), "
               "('C1=CC=C2C(=C1)C(=CC3=CN=CN3)C(=O)N2', 'No indole skeleton "
               "found'), ('C=1C2=C(C=C3C1NC(C3=O)=O)CCC2', 'No indole skeleton "
               "found'), ('O=C1N(CC=2C1=C(C(O)=C(O)C2O)C)CCC(=O)OC', 'No "
               "indole skeleton found'), "
               "('S1S[C@]23N([C@@H]4[C@@H](O)[C@@H](OC)CC([C@@H]4C2)=O)C([C@]15N([C@@H]6[C@@H](O)C=CC([C@H]6C5)=O)C3=O)=O', "
               "'No indole skeleton found'), "
               "('ClC1=C[C@H]2C(=O)[C@]([C@@]1(C=C)C)([N+]#[C-])C3=C4C(=CC=C3)N(C([C@@H]4C2(C)C)=O)C', "
               "'No indole skeleton found'), "
               "('O=C1N(CC2=C1C=C(O)C3=C2OC(C(O)C3)(CC/C=C(/CCC=C(C)C)\\\\C)C)CCCCC(=O)O', "
               "'No indole skeleton found'), "
               "('C1CN2C3=NC4=CC=CC=C4C3=NN=C2SC1', 'No indole skeleton "
               "found'), ('CC(COC)NC1=NNC(=C2C=NC3=CC=CC=C32)CS1', 'No indole "
               "skeleton found'), "
               "('COC(=O)[C@@H]1C[C@@]23CC[C@@]11N(C(=O)OC)c4ccccc4[C@]1(CCN2C(=O)C=C3)C(=O)OC', "
               "'No indole skeleton found'), "
               "('C[C@@H]1[C@H]([C@@H](O[C@]12C3=C(C=CC(=C3)I)N(C2=O)CC=C)CCO)[Si](C)(C)C4=CC=C(C=C4)OC', "
               "'No indole skeleton found'), "
               "('C1=CC2=C(C=C1Br)C(=CNNC(=O)C3=CC=NC=C3)C=N2', 'No indole "
               "skeleton found'), "
               "('CC1=C(NC(=C1C(=O)N2CCC[C@@H]2CN3CCCC3)C)C=C4C5=C(C=CC(=C5)S(=O)(=O)CC6=C(C=CC=C6Cl)Cl)NC4=O', "
               "'No indole skeleton found'), "
               "('CC1=CC=C(C=C1)C2=CSC(=C3C=NC4=CC=CC=C43)N2', 'No indole "
               "skeleton found'), "
               "('O=C(O)C1=CC2=C(N[C@@H](C2)[C@](O)(CCC(=C(C)C)C)C)C=C1', 'No "
               "indole skeleton found'), "
               "('C1=CC2=C(C=C1I)C(=CC3=CC(=C(C(=C3)Br)O)Br)C(=O)N2', 'No "
               "indole skeleton found'), "
               "('[H][C@@]12CCCN1C(=O)C1=C[C@]3([C@H](C=C(C)C)N1C2=O)C(=O)Nc1cc(OC)ccc31', "
               "'No indole skeleton found'), "
               "('CNC(=O)Oc1ccc2N(C)[C@H]3OCC[C@@]3(C)c2c1', 'No indole "
               "skeleton found'), "
               "('N1C(C(C=2C=CC=CC12)=CC=3NC(=C(C3C)CCC(=O)O)C)=O', 'No indole "
               "skeleton found'), ('O=C1N(C=2C=C(OC)C=CC2[C@]1(O)CC(=O)C)C', "
               "'No indole skeleton found'), "
               "('CC[C@H](C)C(=O)C(=O)N[C@H]1Cc2ccc(O)c(c2)-c2cccc3c2NC(=O)[C@@]3(O)[C@H](O)[C@H](NC(=O)[C@H](CC(N)=O)NC1=O)C(=O)N\\\\C=C/C', "
               "'No indole skeleton found'), "
               "('CC1CC2=C(N1S(=O)(=O)C)C=CC(=C2)C3=CSC(=N3)NC(=O)CC4=CC=CS4', "
               "'No indole skeleton found'), "
               "('C1=CC=C(C=C1)CN(CC2=CC=CC=C2)CC3=NNC(=C4C=C5C=CC=CC5=N4)S3', "
               "'No indole skeleton found'), ('O=C1CNc2ccccc12', 'No indole "
               "skeleton found'), ('CSc1cc2CCN(C(=O)Nc3cccnc3)c2cc1C(F)(F)F', "
               "'No indole skeleton found'), "
               "('CCCCN1C2=CC=CC=C2C(=C(C#N)C(=O)N)C1=O', 'No indole skeleton "
               "found'), "
               "('CC(\\\\C=C\\\\C=C1\\\\N(CCS(O)(=O)=O)c2ccc(cc2C1(C)C)S(O)(=O)=O)=C/C=C/C1=[N+](CCS([O-])(=O)=O)c2ccc(cc2C1(C)C)S(O)(=O)=O', "
               "'No indole skeleton found'), "
               "('CC1=CC(=C(O1)C)C(=O)OCC(=O)N2CCC3=CC=CC=C32', 'No indole "
               "skeleton found'), ('O=C1C(=O)C=C(NCCCO)C2=C1NC=C2CCO', 'No "
               "indole skeleton found'), "
               "('N1C(/C(/C=2C=CC=CC12)=C\\\\C=3NC(=CC3C)C)=O', 'No indole "
               "skeleton found'), "
               "('CN(C1=C(C=CC=N1)CNC2=NC(=NC=C2C(F)(F)F)NC3=CC4=C(C=C3)NC(=O)C4)S(=O)(=O)C', "
               "'No indole skeleton found'), "
               "('O=C1N(CC=2C1=C(C(O)=C(O)C2O)C)CCCCC(=O)OC', 'No indole "
               "skeleton found'), "
               "('S1S[C@]23N([C@@H]4[C@@H](O)[C@@H](O)CC([C@@H]4C2)=O)C([C@]15N([C@@H]6[C@@H](O)[C@@H](O)CC([C@@H]6C5)=O)C3=O)=O', "
               "'No indole skeleton found'), ('OC(=O)C1CC2=CC=CC=C2N1', 'No "
               "indole skeleton found'), "
               "('S(=O)(=O)(C1=CC=C2C(=C1)C(C(N2)=O)=O)N3CCN(CC3)C', 'No "
               "indole skeleton found'), ('C=1C=CC=C2NC(C(C12)CC(=O)O)=O', 'No "
               "indole skeleton found'), ('CN(C)CCN1CCC2=CC=CC=C21', 'No "
               "indole skeleton found'), "
               "('[K+].[K+].CN1c2ccc(NC(=O)CI)cc2C(C)(C)\\\\C1=C\\\\C=C\\\\C=C\\\\C1=[N+](CCS([O-])(=O)=O)c2ccc3c(cc(cc3c2C1(C)C)S([O-])(=O)=O)S([O-])(=O)=O', "
               "'No indole skeleton found'), "
               "('CCOC1=C(C=C(C=C1)C2CC3=C(C(=C(N3)C(=O)OC)C)C(=O)C2)OC', 'No "
               "indole skeleton found'), "
               "('O1N(CC[C@@]2([C@]1(N(C3=C2C=C(OC(=O)NCCCCCCC)C=C3)C)[H])C)C', "
               "'No indole skeleton found'), "
               "('O1N=C(C(OC)=C)CC2(C1NC=3C(=CC=CC23)/C=C/C(=C)C)O', 'No "
               "indole skeleton found'), "
               "('ClC1=C[C@H]2C(=O)[C@]([C@@]1(C=C)C)(N=C=S)C3=C4C(=CC=C3)N(C([C@@H]4C2(C)C)=O)C', "
               "'No indole skeleton found'), "
               "('CC1=CC=CC=C1CN2C3=CC=CC=C3C(=NNC(=S)NC4=CC=CC=C4F)C2=O', 'No "
               "indole skeleton found'), "
               "('COC(=O)c1ccc2\\\\C(=C(\\\\Nc3ccc(cc3)N(C)C(=O)CN3CCN(C)CC3)c3ccccc3)C(=O)Nc2c1', "
               "'No indole skeleton found'), "
               "('[H]C(=CC([H])=C1N(CCCCCC(=O)ON2C(=O)CCC2=O)c2ccc(cc2C1(C)C)S([O-])(=O)=O)C1=[N+](CCCCCC(=O)ON2C(=O)CCC2=O)c2ccc(cc2C1(C)C)S([O-])(=O)=O', "
               "'No indole skeleton found'), "
               "('S1SC23N(C4C(C2)=COC=CC4O)C(=O)C15N(C6C(C5)=COC=CC6OC(=O)C)C3=O', "
               "'No indole skeleton found'), "
               "('CCc1c(C)[nH]c2CCC(CN3CCOCC3)C(=O)c12', 'No indole skeleton "
               "found'), ('Clc1cc2NC(=O)Cc2cc1CCN1CCN(CC1)c1nsc2ccccc12', 'No "
               "indole skeleton found'), ('O=C/1NC=2C=CC=CC2\\\\C1=C\\\\C', "
               "'No indole skeleton found')]",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 213,
    'num_false_positives': 100,
    'num_true_negatives': 2890,
    'num_false_negatives': 15,
    'num_negatives': None,
    'precision': 0.6805111821086262,
    'recall': 0.9342105263157895,
    'f1': 0.787430683918669,
    'accuracy': 0.9642635177128651}


"""
Classifies: CHEBI:33238 monoatomic entity
"""
from rdkit import Chem

def is_monoatomic_entity(smiles: str):
    """
    Determines if a molecule is a monoatomic entity (consists of a single atom).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a monoatomic entity, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule consists of a single atom
    if mol.GetNumAtoms() != 1:
        return False, "Molecule has more than one atom"

    # Check if the atom has a charge or is an isotope
    atom = mol.GetAtomWithIdx(0)
    charge = atom.GetFormalCharge()
    isotope = atom.GetIsotope()

    if charge != 0 or isotope != 0:
        atom_symbol = atom.GetSymbol()
        if charge != 0:
            charge_str = "+" * charge if charge > 0 else "-" * abs(charge)
            return True, f"{atom_symbol}({charge_str})"
        else:
            return True, f"{atom_symbol}-{isotope}"
    else:
        return True, f"Neutral {atom.GetSymbol()} atom"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33238',
                          'name': 'monoatomic entity',
                          'definition': 'A monoatomic entity is a molecular '
                                        'entity consisting of a single atom.',
                          'parents': ['CHEBI:33259']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 18,
    'num_false_positives': 100,
    'num_true_negatives': 43746,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.15254237288135594,
    'recall': 1.0,
    'f1': 0.2647058823529412,
    'accuracy': 0.9977202261535656}


"""
Classifies: CHEBI:24780 imidazoles
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_imidazoles(smiles: str):
    """
    Determines if a molecule contains an imidazole ring.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule contains imidazole ring, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
    
    # SMARTS patterns for imidazole ring and its variations
    imidazole_patterns = [
        "[nH,n]1c[nH,n]cc1",  # Basic imidazole pattern
        "N1C=NC=C1",  # Alternative representation
        "N1C=CN=C1",  # Alternative representation
        "[nH]1cncc1",  # Another representation
        "n1cncc1",     # N-substituted version
        "N1=CN=CC1",   # Another tautomer
        "N1=CN=C[CH]1", # Another form
        "[nH]1c(*)ncc1", # Substituted version
        "N1=C(*)N=CC1",  # Another substituted form
        "[nH]1cnc(*)c1", # 4-substituted form
        "N1=CNC(*)=C1",  # Another 4-substituted form
        "C1=CN=CN1",     # Another representation
        "[n]1c[n]cc1",   # Aromatic form
        "N1=C([*,H])N=C([*,H])C1([*,H])", # General substituted form
    ]
    
    # Check for presence of imidazole ring
    found = False
    matching_pattern = None
    for pattern in imidazole_patterns:
        pattern_mol = Chem.MolFromSmarts(pattern)
        if pattern_mol and mol.HasSubstructMatch(pattern_mol):
            found = True
            matching_pattern = pattern
            break
            
    if not found:
        return False, "No imidazole ring found"
    
    # Get atoms involved in the imidazole ring
    matches = mol.GetSubstructMatches(Chem.MolFromSmarts(matching_pattern))
    if not matches:
        return False, "No imidazole ring found"
        
    ring_atoms = set(matches[0])
    
    # Check for substituents and their types
    substituents = set()
    for atom_idx in ring_atoms:
        atom = mol.GetAtomWithIdx(atom_idx)
        for neighbor in atom.GetNeighbors():
            if neighbor.GetIdx() not in ring_atoms:
                symbol = neighbor.GetSymbol()
                if symbol != 'H':  # Ignore hydrogens
                    substituents.add(symbol)
    
    # Additional check for fused ring systems
    ring_info = mol.GetRingInfo()
    ring_atoms_list = list(ring_atoms)
    is_fused = False
    for ring in ring_info.AtomRings():
        if any(atom_idx in ring_atoms_list for atom_idx in ring) and \
           not all(atom_idx in ring_atoms_list for atom_idx in ring):
            is_fused = True
            break
    
    if is_fused:
        return True, "Fused imidazole ring system"
    elif substituents:
        return True, f"Imidazole with substituents: {', '.join(sorted(substituents))}"
    else:
        return True, "Unsubstituted imidazole"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24780',
                          'name': 'imidazoles',
                          'definition': 'A five-membered organic heterocycle '
                                        'containing two nitrogen atoms at '
                                        'positions 1 and 3, or any of its '
                                        'derivatives; compounds containing an '
                                        'imidazole skeleton.',
                          'parents': ['CHEBI:23677']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.4421052631578947 is too low.\n'
               "True positives: [('N=1C(=C(NC1)C)C(=O)O', 'Imidazole with "
               "substituents: O'), ('OC(=O)C(=O)Cc1c[nH]cn1', 'Imidazole with "
               "substituents: C'), ('OCc1c[nH]cn1', 'Imidazole with "
               "substituents: O'), "
               "('C1=CC=C(C=C1)C2=C(N3C(=O)C(=CC4=CC(=CC=C4)O)SC3=N2)C5=CC=CC=C5', "
               "'Imidazole with substituents: C'), ('ClCc1nccn1Cc1ccccc1', "
               "'Imidazole with substituents: C, Cl'), "
               "('Clc1ccc([C@@H](Cn2ccnc2)OCc2csc3c(Cl)cccc23)c(Cl)c1', "
               "'Imidazole with substituents: C'), "
               "('C(=C(/C(OCC)=O)\\\\C(C)=O)\\\\C=1N(C)C(=CN1)[N+]([O-])=O', "
               "'Imidazole with substituents: C, O'), "
               "('O[N+]([O-])=O.Clc1ccc([C@H](Cn2ccnc2)OCc2csc3c(Cl)cccc23)c(Cl)c1', "
               "'Imidazole with substituents: C'), ('OCCC=1NC=NC1', 'Imidazole "
               "with substituents: C'), "
               "('CN1C=C(N=C1)S(=O)(=O)N2C[C@@H](COC[C@@H]3[C@@H]2CC[C@@H](O3)CC(=O)OC)O', "
               "'Imidazole with substituents: N, C, O'), "
               "('O1[C@@H](N2C=C(N=C2)CC(=O)O)[C@H](O)[C@H](O)[C@H]1CO', "
               "'Imidazole with substituents: C'), "
               "('OC=1C(C=2N(CC3=CC=CC=C3)C=NC2)=CC=CC1', 'Imidazole with "
               "substituents: C'), ('NC(=N)Nc1nc2ccccc2[nH]1', 'Imidazole with "
               "substituents: C'), "
               "('CC1=CN(C=N1)C1=CCC2C3CC=C4C[C@H](CC[C@]4(C)C3CC[C@]12C)NC(=O)C1=CC=C(F)C=C1', "
               "'Imidazole with substituents: C'), "
               "('C/1CC2=C(\\\\C1=N/O)C=CC(=C2)C=3N=C(NC3C=4C=CN=CC4)C5=CC=C(C=C5)OCCN(C)C', "
               "'Imidazole with substituents: C'), "
               "('O1[C@@H]([C@H]([C@H]([C@@H]1N2C(=CN=C2)N)O)O)COP(=O)(O)O', "
               "'Imidazole with substituents: C'), ('O=C(O)C=1NC(C)=NC1', "
               "'Imidazole with substituents: O'), "
               "('C1=CN2C=C(N=C2N=C1)C3=CC(=C(C=C3)Cl)Cl', 'Imidazole with "
               "substituents: C'), ('Cn1cnc(c1)C(O)=O', 'Imidazole with "
               "substituents: C, O'), "
               "('COC1=C(C=CC(=C1)SC)C2=NC3=C(N2)C=NC=C3', 'Imidazole with "
               "substituents: N, C'), "
               "('CC[C@H](N(Cc1ccco1)C(=O)n1ccnc1)C(=O)OCCCC=C', 'Imidazole "
               "with substituents: C'), "
               "('CC(=O)N1CCN(CC1)c1ccc(OC[C@H]2CO[C@](Cn3ccnc3)(O2)c2ccc(Cl)cc2Cl)cc1', "
               "'Imidazole with substituents: C'), "
               "('CC(=O)N1CCN(CC1)c1ccc(OCC2COC(Cn3ccnc3)(O2)c2ccc(Cl)cc2Cl)cc1', "
               "'Imidazole with substituents: C'), ('S1C=2N(C=C(N2)CO)C=C1', "
               "'Imidazole with substituents: C, O'), "
               "('O1[C@@H](N2C(N)=C(N=C2)NC(O)=O)[C@H](O)[C@H](O)[C@H]1CO', "
               "'Imidazole with substituents: C'), "
               "('Clc1ccc(C(Cn2ccnc2)OCC=C)c(Cl)c1', 'Imidazole with "
               "substituents: C'), ('Clc1ccc(CNC(=N)SCCCc2c[nH]cn2)cc1', "
               "'Imidazole with substituents: C'), "
               "('Nc1nccc(n1)-c1c(ncn1C1CCNCC1)-c1ccc(F)cc1', 'Imidazole with "
               "substituents: N, C'), "
               "('COC1=CC=C(C=C1)C2=CN(C3=[N+]2CCCCC3)C4=CC=C(C=C4)Cl', "
               "'Imidazole with substituents: C'), ('CN1C(=NC=C1)C(O)=O', "
               "'Imidazole with substituents: C, O'), "
               "('O=C(OC)/C=C/C=1N=CN(C1CC=C(C)C)C', 'Imidazole with "
               "substituents: C'), ('CC(c1c[nH]cn1)c1cccc(C)c1C', 'Imidazole "
               "with substituents: C'), ('C[C@H](N)c1nc(C)c(O)n1CC(O)=O', "
               "'Imidazole with substituents: N, C'), "
               "('CC1=CC=CC2=NC(=C(N12)NCC3CCCO3)C4=CC(=C(C=C4)O)OC', "
               "'Imidazole with substituents: C'), "
               "('COc1ccc(CN([C@@H](C)c2nc(c[nH]2)-c2ccccc2)C(=O)[C@@H](N)Cc2c(C)cc(cc2C)C(N)=O)cc1C(O)=O', "
               "'Imidazole with substituents: C, N'), "
               "('NCCCC(=O)N[C@@H](Cc1c[nH]cn1)C(O)=O', 'Imidazole with "
               "substituents: C'), "
               "('C(C(=O)C(N1C=CN=C1)OC2=CC=C(C=C2)Cl)(C)(C)C', 'Imidazole "
               "with substituents: C'), ('C=1(N(C(=NC1)C)CC(C)O)[N+]([O-])=O', "
               "'Imidazole with substituents: C, O'), "
               "('CC1=CC(=NO1)NC(=O)CSC2=NC=CN2C3=CC(=CC=C3)OC', 'Imidazole "
               "with substituents: C'), ('Cc1ccc2[nH]cnc2c1', 'Imidazole with "
               "substituents: C'), "
               "('CN1C(=CN=C1NCC2=C(C=CC(=C2)Cl)O)C3=CC=CC=C3', 'Imidazole "
               "with substituents: C'), "
               "('N(=N/[N+]=1C(=C(N2C1C=CC=C2)C)C3=CC=CC=C3)\\\\[N+]=4C(=C(N5C4C=CC=C5)C)C6=CC=CC=C6', "
               "'Imidazole with substituents: C, N')]\n"
               'False positives: '
               "[('C1=CC=C2C(=C1)N=C(S2)NC(=O)COC(=O)CC3=CN4C=CSC4=N3', "
               "'Imidazole with substituents: C'), "
               "('O=C(N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC=3NC=NC3)CC(O)=O', "
               "'Imidazole with substituents: C'), "
               "('O=C(N[C@@H](C(C)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC=1NC=NC1)C(C)C', "
               "'Imidazole with substituents: C'), "
               "('C1=CC=C2C(=C1)N=C(N2CC(=O)NC3=CC=CC=C3F)C4=CSC=N4', "
               "'Imidazole with substituents: N, C'), "
               "('CCCCC\\\\C=C/CCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'Imidazole with substituents: N, C'), "
               "('CCCCCCN1C2=C(N=C1N3CCN(CC3)C4=CC=CC=C4)N(C(=O)NC2=O)C', "
               "'Imidazole with substituents: N, C, O'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC(C)([C@H](C(NCCC(NCCSC(=O)/C=C/C/C=C/CCCCCCCC)=O)=O)O)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'Imidazole with substituents: N, C'), "
               "('O=C(N[C@@H](CC=1NC=NC1)C(O)=O)[C@@H](NC(=O)[C@@H](N)C(C)C)C(C)C', "
               "'Imidazole with substituents: C'), "
               "('C[C@@H]1O[C@@H](OCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP(O)(O)=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'Imidazole with substituents: N, C'), "
               "('[C@@H]1(O[C@H]([C@@H]([C@@H]1O*)O)*)COP(O[C@@H]2[C@H](O[C@@H](N3C=4N=C(NC(=O)C4N=C3)N)[C@@H]2O)COP(=O)([O-])[O-])(=O)[O-]', "
               "'Imidazole with substituents: N, C, O'), "
               "('SC[C@H](NC(=O)[C@@H](N)CC=1NC=NC1)C(=O)NCC(O)=O', 'Imidazole "
               "with substituents: C'), "
               "('S(CC[C@H](NC(=O)[C@H]1NCCC1)C(=O)N[C@@H](CC=2NC=NC2)C(O)=O)C', "
               "'Imidazole with substituents: C'), "
               "('CC(=O)N[C@@H](Cc1cnc[nH]1)C([O-])=O', 'Imidazole with "
               "substituents: C'), "
               "('O=C(N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(O)=O)CC=3NC=NC3', "
               "'Imidazole with substituents: C'), "
               "('P(OP(OP(OC[C@H]1O[C@@H](N2C=3N=CN=C(N)C3N=C2)[C@@H]([C@@H]1O)OP(=O)(OC[C@H]4O[C@H]([C@@H]([C@@H]4O)O)N5C=6N=CN=C(N)C6N=C5)O)(=O)O)(=O)O)(=O)(O)O', "
               "'Imidazole with substituents: N, C'), "
               "('CC(C)(COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)\\\\C=C\\\\CCCCCCCCCCC(O)=O', "
               "'Imidazole with substituents: N, C'), "
               "('C(=C/C(SCCNC(CCNC(=O)[C@@H](C(COP(OP(OC[C@H]1O[C@@H](N2C3=C(C(=NC=N3)N)N=C2)[C@@H]([C@@H]1OP([O-])([O-])=O)O)(=O)[O-])(=O)[O-])(C)C)O)=O)=O)\\\\CCCC[C@H](O[C@@H]4O[C@@H](C)[C@@H](C[C@H]4O)OC(C=5C=6C=CC=CC6NC5)=O)C', "
               "'Imidazole with substituents: N, C'), "
               "('C1=CC=C(C=C1)COC(=O)CC2=CN3C=CSC3=N2', 'Imidazole with "
               "substituents: C'), "
               "('CCCCCCCCCCCCCCCCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'Imidazole with substituents: N, C'), "
               "('CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O[C@@H]2O[C@H](COP([O-])([O-])=O)[C@@H](O)[C@H]2O)[C@@H]1O)n1cnc2c(N)ncnc12', "
               "'Imidazole with substituents: N, C'), "
               "('S=C1N(C2OC(C(O)C2O)CO)C3=NC=NC(N)=C3N1', 'Imidazole with "
               "substituents: N, C, S'), "
               "('O=C(N[C@@H](CC=1NC=NC1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(=O)N)CC=2C=3C(NC2)=CC=CC3', "
               "'Imidazole with substituents: C'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'Imidazole with substituents: N, C'), "
               "('CC(C)(COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CCCCC([O-])=O', "
               "'Imidazole with substituents: N, C'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Imidazole with substituents: N, C'), "
               "('Nc1ncnc2n(cnc12)[C@@H]1O[C@H](COP(O)(=O)OP(O)(O)=O)[C@@H](O)[C@H]1O', "
               "'Imidazole with substituents: C, N'), "
               "('C[C@H](CCCCCCCCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'Imidazole with substituents: N, C'), "
               "('C[C@H](CCCCCCCCCCCCCC/C=C/C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'Imidazole with substituents: N, C'), "
               "('Cn1cnc2n(cnc2c1=O)[C@@H]1O[C@H](COP(O)(=O)O[C@@H]2[C@@H](COP(O)(=O)O[C@@H]3[C@@H](COP(O)(=O)O[C@@H]4[C@@H](CO)O[C@H]([C@@H]4O)n4cnc5c(O)ncnc45)O[C@H]([C@@H]3O)n3cnc4c3nc(N)[nH]c4=O)O[C@H]([C@@H]2O)n2ccc(N)nc2=O)[C@@H](OP(O)(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP(O)(O)=O)n2ccc(=O)[nH]c2=O)[C@H]1O', "
               "'Imidazole with substituents: N, C, O'), "
               "('O1C(O)[C@H](O)[C@H](OC(C)=O)[C@H]1COP(OP(OC[C@@H]2[C@H]([C@H]([C@H](N3C4=NC=NC(=C4N=C3)N)O2)O)O)(=O)[O-])(=O)[O-]', "
               "'Imidazole with substituents: N, C'), "
               "('O=C(O)C(NC(=O)C(N)CCCN=C(N)N)C1OC(N2C3=NC(=NC=C3N=C2)N)C4OCC(C4(C1O)O)O', "
               "'Imidazole with substituents: N, C'), "
               "('O=C(N[C@@H](CCC(=O)N)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC1=CC=CC=C1)CC=2NC=NC2', "
               "'Imidazole with substituents: C'), ('OC(=O)c1nc2ccccc2[nH]1', "
               "'Imidazole with substituents: C, O'), "
               "('CN(C)S(=O)(=O)C1=CC=C(C=C1)C(=O)NC2=NC3=CC=CC=C3N2', "
               "'Imidazole with substituents: C'), "
               "('SCC(NC(=O)C(NC(=O)C(NC(=O)C(NC(=O)C(NC(=O)C1N(C(=O)C(NC(=O)C(N)CCC=O)CC(=O)N)CCC1)CCCCN)C(C)C)C)CC=2NC=NC2)C(=O)NC(C(=O)NC(C(=O)NC(C(=O)N)CCC(=O)N)CO)C', "
               "'Imidazole with substituents: C'), "
               "('O=C(N[C@@H](CC=1NC=NC1)C(O)=O)[C@@H](NC(=O)[C@@H](N)C)CC2=CC=CC=C2', "
               "'Imidazole with substituents: C'), "
               "('CC1=C(C(=O)N(N1C)C2=CC=CC=C2)NCC3=NC4=CC=CC=C4N3', "
               "'Imidazole with substituents: C, N'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Imidazole with substituents: C, N'), "
               "('[H][C@]12[C@H](CC(N)=O)[C@@]3(C)CCC(=O)NC[C@@H](C)OP([O-])(=O)O[C@@H]4[C@@H](COP([O-])([O-])=O)O[C@@H]([C@@H]4O)n4c[n+](c5cc(C)c(C)cc45)[Co-3]456(C[C@H]7O[C@H]([C@H](O)[C@@H]7O)n7cnc8c(N)ncnc78)N1C3=C(C)C1=[N+]4C(=CC3=[N+]5C(=C(C)C4=[N+]6[C@]2(C)[C@@](C)(CC(N)=O)[C@@H]4CCC(N)=O)[C@@](C)(CC(N)=O)[C@@H]3CCC(N)=O)C(C)(C)[C@@H]1CCC(N)=O', "
               "'Imidazole with substituents: N, C, Co'), "
               "('OC1CN(CCC1)C=2N(C3=C(N2)N(C(=O)N(C3=O)CC4=NC=5C(C(=N4)C)=CC=CC5)C)CC#CC', "
               "'Imidazole with substituents: N, C, O'), "
               "('NC(=O)c1ccc[n+](c1)[C@@H]1O[C@H](COP([O-])(=O)OP([O-])(=O)OC[C@H]2O[C@H]([C@H](OP([O-])([O-])=O)[C@@H]2O)n2cnc3c(N)ncnc23)[C@@H](O)[C@H]1O', "
               "'Imidazole with substituents: N, C'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCC(O)CC(=O)SCCNC(=O)CCNC(=O)C(O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'Imidazole with substituents: N, C'), "
               "('N1(C2=C(C(=NC=N2)N)N=C1)[C@@H]3O[C@H](COP([O-])(OC(CC[NH3+])=O)=O)[C@H]([C@H]3O)O', "
               "'Imidazole with substituents: N, C'), "
               "('O=C(N[C@@H](CCCN=C(N)N)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC=1NC=NC1)CCCN=C(N)N', "
               "'Imidazole with substituents: C'), "
               "('O=C(N[C@@H](CC=1NC=NC1)C(=O)N[C@@H](CCC(=O)N)C(O)=O)[C@@H](N)CCCCN', "
               "'Imidazole with substituents: C'), "
               "('O=C(N[C@@H](CC(C)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC=1NC=NC1)CC(=O)N', "
               "'Imidazole with substituents: C'), "
               "('CC(C)(COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)CC(=O)C[N+](C)(C)C', "
               "'Imidazole with substituents: N, C'), "
               "('C\\\\C=C/CCCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'Imidazole with substituents: N, C'), "
               "('O[N+]([O-])=O.Clc1ccc([C@@H](Cn2ccnc2)OCc2c(Cl)cccc2Cl)c(Cl)c1', "
               "'Imidazole with substituents: C'), "
               "('CC(C)[C@H](NC(=O)[C@@H](N)Cc1c[nH]cn1)C(O)=O', 'Imidazole "
               "with substituents: C'), "
               "('NC=1N=CN=C2C1N=CN2[C@@H]3O[C@H](CO)CC3', 'Imidazole with "
               "substituents: C, N'), "
               "('CC1=C(N2C=CC=CC2=N1)C(C3=CC=CC=C3)(C4=CC=CC=C4)O', "
               "'Imidazole with substituents: C, O'), "
               "('CN(C)c1ncnc2n(cnc12)[C@@H]1O[C@H](COP(=O)(NCCCC[C@H](NC(C)=O)C(O)=O)N2CCOCC2)[C@@H](O)[C@H]1O', "
               "'Imidazole with substituents: C, N'), "
               "('CCCCCCCCCCC(C(O)=O)C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'Imidazole with substituents: N, C'), "
               "('C[C@@H]1O[C@@H](OCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP(O)(O)=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'Imidazole with substituents: N, C'), "
               "('[C@@H]1(N2C3=C(C(=NC=N3)N)N=C2)O[C@H](COP(OP(OCC([C@H](C(NCCC(NCCSC(=O)CCC/C=C\\\\C[C@@H]4[C@@H](/C=C/[C@@H](O)CCCCC)[C@@H](C[C@@H]4O)O)=O)=O)O)(C)C)(=O)[O-])(=O)[O-])[C@H]([C@H]1O)OP([O-])([O-])=O', "
               "'Imidazole with substituents: N, C'), "
               "('CC1=CC=C(C=C1)C(=O)CSC2=NC=CN2C3=CC=CC=C3', 'Imidazole with "
               "substituents: C'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Imidazole with substituents: C, N'), "
               "('C[C@@H]1O[C@@H](OCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2OP(O)(O)=O)N3C=NC4=C(N)N=CN=C34)[C@H](O)C[C@H]1O', "
               "'Imidazole with substituents: N, C'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Imidazole with substituents: N, C'), "
               "('OC(=O)[C@@H](NC(=O)CCCN)CC=1N(C=NC1)C', 'Imidazole with "
               "substituents: C'), "
               "('O=C(N[C@@H](CCCN=C(N)N)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC1=CC=CC=C1)CC=2NC=NC2', "
               "'Imidazole with substituents: C'), "
               "('O=C(N[C@@H]([C@H](O)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC=1NC=NC1)CCCN=C(N)N', "
               "'Imidazole with substituents: C'), "
               "('C[C@H](CCCCCCCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)N2C=NC3=C(N)N=CN=C23)O[C@@H]4O[C@@H](C)[C@H](O)C[C@H]4O', "
               "'Imidazole with substituents: N, C'), "
               "('C1([C@@H]([C@H]1C=C(C)C)C(=O)SCCNC(CCNC(=O)[C@@H](C(COP(OP(OC[C@H]2O[C@@H](N3C4=C(C(=NC=N4)N)N=C3)[C@@H]([C@@H]2OP([O-])([O-])=O)O)(=O)[O-])(=O)[O-])(C)C)O)=O)(C)C', "
               "'Imidazole with substituents: C, N'), "
               "('C1(=O)NC(=NC2=C1[N+](=CN2[C@@H]3O[C@H](COP(OP(OP(OC[C@H]4O[C@H]([C@@H]([C@@H]4O*)O)N5C6=C(C(=NC=N6)N)N=C5)(=O)[O-])(=O)[O-])(=O)[O-])[C@@H](O)[C@H]3O)C)N(C)C', "
               "'Imidazole with substituents: C, N, O'), "
               "('O=C(NC(CC1=CC=CC=C1)C(O)=O)C(N)CC=2NC=NC2', 'Imidazole with "
               "substituents: C'), "
               "('O[C@@H]1CC=2[C@@]([C@@]3([C@]([C@]4([C@](CC3)(C(N5C=6C(N=C5)=CC=CC6)=CC4)C)[H])(CC2)[H])[H])(CC1)C', "
               "'Imidazole with substituents: C'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Imidazole with substituents: N, C'), "
               "('C[C@@H]1OC(OP(O)(=O)OP(O)(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2O)n2cnc3c2nc(N)[nH]c3=O)[C@@H](O)[C@H](O)C1=O', "
               "'Imidazole with substituents: N, C, O'), "
               "('O[C@@H]([C@H](NC(=O)[C@@H](N)CC=1NC=NC1)C(=O)N[C@@H](CC(=O)N)C(O)=O)C', "
               "'Imidazole with substituents: C'), "
               "('C1(=CC=C(C=C1)CN2C3=C(C(=O)[O-])C=CC=C3N=C2OCC)C=4C=CC=CC4C=5N=NN(N5)[C@H]6[C@@H]([C@H]([C@@H]([C@H](O6)C([O-])=O)O)O)O', "
               "'Imidazole with substituents: C'), "
               "('CN1C2=C(C(=O)N(C1=O)C)N(C(=N2)NN=CC3=CC=CS3)CC4=CC=CC=C4', "
               "'Imidazole with substituents: N, C, O'), "
               "('Nc1ncnc2n(cnc12)[C@@H]1O[C@H](COP(O)(=O)OP(O)(=O)OP(O)(=O)OP(O)(=O)OP(O)(=O)OP(O)(=O)OC[C@H]2O[C@H]([C@H](O)[C@@H]2O)n2cnc3c(N)ncnc23)[C@@H](O)[C@H]1O', "
               "'Imidazole with substituents: C, N'), "
               "('O=C(N[C@@H]([C@H](CC)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(C)C)CC=1NC=NC1', "
               "'Imidazole with substituents: C'), "
               "('C1(=O)NC(=NC2=C1[N+](=CN2[C@@H]3O[C@H](COP(OP(OP(OC[C@H]4O[C@H]([C@@H]([C@@H]4OP(OC[C@H]5O[C@H]([C@@H]([C@@H]5O*)OC)*)(=O)[O-])OC)*)(=O)[O-])(=O)[O-])(=O)[O-])[C@@H](O)[C@H]3O)C)N', "
               "'Imidazole with substituents: C, N, O'), "
               "('CC[C@H](C)[C@@H](C(=O)N[C@@]1(CCCC=CCCC[C@](NC(=O)[C@@H](NC(=O)[C@@H](NC(=O)[C@@H](NC1=O)CC(C)C)CS)CCCNC(=N)N)(C)C(=O)N[C@@H](CC2=CNC=N2)C(=O)N[C@@H](CC3=CNC=N3)C(=O)N[C@@H](CO)C(=O)N[C@@H]([C@@H](C)O)C(=O)N)C)NC(=O)[C@H](CCCNC(=N)N)NC(=O)[C@H](CCCNC(=N)N)NC(=O)[C@H](CCCNC(=N)N)NC(=O)[C@H](CC(C)C)NC(=O)[C@H](CCCNC(=N)N)NC(=O)[C@H](CCC(=O)O)NC(=O)[C@H](CC4=CNC5=CC=CC=C54)NC(=O)CCNC(=O)C', "
               "'Imidazole with substituents: C'), "
               "('C(CN(C)C)N1C2=C(N=C1)N(C)C(N(C2=O)C)=O', 'Imidazole with "
               "substituents: N, C, O'), "
               "('SC[C@H](NC(=O)[C@@H](N)CC=1NC=NC1)C(=O)N[C@@H](CC(O)=O)C(O)=O', "
               "'Imidazole with substituents: C'), "
               "('[H+].Cc1ncc(n1CCO)[N+]([O-])=O', 'Imidazole with "
               "substituents: C, O'), "
               "('CC=CC1=CN=C2C(=C1)C(=O)N(C[C@H]([C@H](O2)CN(C)S(=O)(=O)C3=CN(C=N3)C)C)[C@@H](C)CO', "
               "'Imidazole with substituents: C, N, O'), "
               "('FC(F)(F)C=1C=CC(N2C=CN=C2)=NC1', 'Imidazole with "
               "substituents: C'), "
               "('CN1C=C(N=C1)S(=O)(=O)N[C@H]2C=C[C@H](O[C@H]2CO)CC(=O)N3CCN(CC3)C4=CC=CC=C4', "
               "'Imidazole with substituents: N, C, O'), "
               "('CN1C2=CC=CC=C2N=C1C(COC)(C3=CC=CC=C3)N', 'Imidazole with "
               "substituents: N, C'), "
               "('CC1=C(C=C(C=C1)N2C(=O)C(=CNC3=CC4=C(C=C3)NC(=O)N4)C(=N2)C)C', "
               "'Imidazole with substituents: C, O'), "
               "('O[C@@H]1[C@@H](COP([O-])([O-])=O)O[C@H]([C@@H]1O)n1cnc2c(NC(CC([O-])=O)C([O-])=O)ncnc12', "
               "'Imidazole with substituents: N, C'), "
               "('C1(OC(CC1*)*)COP(O)(=O)OC2C(OC(C2)N3C(NC4=C3N=C(NC4=O)N)=O)*', "
               "'Imidazole with substituents: N, C, O'), "
               "('N[C@@H](Cc1nc(I)[nH]c1I)C(O)=O', 'Imidazole with "
               "substituents: C'), "
               "('[H]C(=O)Nc1c(ncn1[C@@H]1O[C@H](COP([O-])([O-])=O)[C@@H](O)[C@H]1O)C(N)=O', "
               "'Imidazole with substituents: N, C, O'), "
               "('[C@@H]1(O[C@H]([C@@H]([C@@H]1OP(OC[C@H]2O[C@H]([C@@H]([C@@H]2O*)O)*)(=O)[O-])O)N3C=4N=C(NC(=O)C4N=C3)N)COP(OP(OP(OC)(=O)[O-])(=O)[O-])(=O)[O-]', "
               "'Imidazole with substituents: N, C, O'), "
               "('COC1=C(C=C(C=C1)C(=O)NC(=CC2=CC=CS2)C3=NC4=CC=CC=C4N3)OC', "
               "'Imidazole with substituents: C, N'), "
               "('CCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'Imidazole with substituents: N, C'), "
               "('CC1=C2N3[C@H]([C@H](CC(N)=O)[C@@]2(C)CCC([O-])=O)[C@]2(C)[N+]4=C([C@@H](CCC(N)=O)[C@]2(C)CC(N)=O)C(C)=C2[N+]5=C(C=C6[N+](=C1[C@@H](CCC(N)=O)C6(C)C)[Co--]345C[C@H]1O[C@H]([C@H](O)[C@@H]1O)n1cnc3c(N)ncnc13)[C@@H](CCC(N)=O)[C@]2(C)CC(N)=O', "
               "'Imidazole with substituents: C, N'), "
               "('C1CCC(CC1)C(=O)NCCC2=NC3=CC=CC=C3N2CCOC4=C(C=CC(=C4)F)F', "
               "'Imidazole with substituents: C'), "
               "('S(=O)(=O)(N1CC[N+]([O-])(CC1)CC)C2=CC(=C(OCC)C=C2)C=3NC(=O)C=4N(N3)C(=NC4C)CCC', "
               "'Imidazole with substituents: N, C, O'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Imidazole with substituents: C, N'), "
               "('C(CC[C@]([C@@]1([C@]2([C@H](C[C@@]3([C@]4(CCC(C=C4C=C[C@]3([C@@]2(CC1)[H])[H])=O)C)[H])O)C)[H])(C)[H])(=O)SCCNC(CCNC(=O)[C@@H](C(COP(OP(OC[C@H]5O[C@@H](N6C7=C(C(=NC=N7)N)N=C6)[C@@H]([C@@H]5OP(O)(O)=O)O)(=O)O)(=O)O)(C)C)O)=O', "
               "'Imidazole with substituents: N, C'), "
               "('S(C1=C2C(N=C(N1)N)=NC=N2)C3=C([N+]([O-])=O)N=CN3C', "
               "'Imidazole with substituents: C, N, O'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Imidazole with substituents: N, C'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Imidazole with substituents: N, C')]\n"
               "False negatives: [('CC1=NC=NC1CSCCNC(=NC)NC#N', 'No imidazole "
               "ring found'), "
               "('C(OC)(C1=C(C=CC(=C1)C)C2=N[C@@](C(N2)=O)(C(C)C)C)=O', 'No "
               "imidazole ring found'), "
               "('C(O)(C1=C(C=CC(=C1)C)C2=N[C@](C(N2)=O)(C(C)C)C)=O', 'No "
               "imidazole ring found'), "
               "('CC(C)NC(=O)N1CC(=O)N(C1=O)c1cc(Cl)cc(Cl)c1', 'No imidazole "
               "ring found'), "
               "('[C@@]1(C(N=C(N1)C2=C(C=CC=N2)C(=O)O)=O)(C)C(C)C', 'No "
               "imidazole ring found'), "
               "('C(OC)(C1=C(C=CC(=C1)C)C2=N[C@](C(N2)=O)(C(C)C)C)=O', 'No "
               "imidazole ring found')]",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 43,
    'num_false_positives': 100,
    'num_true_negatives': 2622,
    'num_false_negatives': 5,
    'num_negatives': None,
    'precision': 0.3006993006993007,
    'recall': 0.8958333333333334,
    'f1': 0.450261780104712,
    'accuracy': 0.9620938628158845}


"""
Classifies: CHEBI:24730 hydroxynitrile
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_hydroxynitrile(smiles: str):
    """
    Determines if a molecule is a hydroxynitrile (contains both a nitrile group and a hydroxy group).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a hydroxynitrile, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for nitrile group (C#N)
    nitrile_pattern = Chem.MolFromSmarts('[C]#N')
    has_nitrile = mol.HasSubstructMatch(nitrile_pattern)

    # Check for hydroxy group (OH)
    hydroxy_pattern = Chem.MolFromSmarts('[OH]')
    has_hydroxy = mol.HasSubstructMatch(hydroxy_pattern)

    if has_nitrile and has_hydroxy:
        # Count occurrences
        nitrile_count = len(mol.GetSubstructMatches(nitrile_pattern))
        hydroxy_count = len(mol.GetSubstructMatches(hydroxy_pattern))
        
        return True, f"Contains {nitrile_count} nitrile group(s) and {hydroxy_count} hydroxy group(s)"
    elif not has_nitrile and not has_hydroxy:
        return False, "Missing both nitrile and hydroxy groups"
    elif not has_nitrile:
        return False, "Missing nitrile group"
    else:
        return False, "Missing hydroxy group"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24730',
                          'name': 'hydroxynitrile',
                          'definition': 'Any nitrile compound which also '
                                        'contains a hydroxy group.',
                          'parents': ['CHEBI:18379']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 4,
    'num_false_positives': 100,
    'num_true_negatives': 19083,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.038461538461538464,
    'recall': 1.0,
    'f1': 0.07407407407407407,
    'accuracy': 0.9947881378016366}


"""
Classifies: CHEBI:26820 sulfates
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_sulfates(smiles: str):
    """
    Determines if a molecule is a sulfate (salt or ester of sulfuric acid).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a sulfate, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
        
    # Look for sulfate group patterns
    sulfate_patterns = [
        'OS(=O)(=O)O',     # Sulfuric acid pattern
        'OS(=O)(=O)[O-]',  # Sulfate anion pattern
        '[O-]S(=O)(=O)[O-]', # Sulfate dianion pattern
        'OS(=O)(=O)OC',    # Sulfate ester pattern
        'OS(=O)(=O)O[C,c]' # Aromatic/aliphatic sulfate ester pattern
    ]
    
    for pattern in sulfate_patterns:
        substructure = Chem.MolFromSmarts(pattern)
        if mol.HasSubstructMatch(substructure):
            # Check if it's an ester
            if 'C' in pattern or 'c' in pattern:
                return True, "Sulfate ester found"
            
            # Check for metal ions indicating salt
            for atom in mol.GetAtoms():
                if atom.GetSymbol() in ['Na', 'K', 'Li', 'Al', 'Cu']:
                    return True, "Sulfate salt found"
                if atom.GetFormalCharge() > 0 and atom.GetAtomicNum() not in [1, 6, 7, 8, 15, 16]:
                    return True, "Sulfate salt found (metal cation present)"
                    
            # Check if sulfate is part of organic molecule
            matches = mol.GetSubstructMatches(substructure)
            for match in matches:
                sulfur_idx = [i for i, idx in enumerate(match) 
                            if mol.GetAtomWithIdx(idx).GetSymbol() == 'S'][0]
                sulfur = mol.GetAtomWithIdx(match[sulfur_idx])
                
                # Look at neighbors of oxygens bonded to sulfur
                for neighbor in sulfur.GetNeighbors():
                    if neighbor.GetSymbol() == 'O':
                        for o_neighbor in neighbor.GetNeighbors():
                            if o_neighbor.GetSymbol() not in ['S', 'H']:
                                return True, "Sulfate ester found"
            
            return True, "Sulfate group found"
            
    return False, "No sulfate group found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26820',
                          'name': 'sulfates',
                          'definition': 'Salts and esters of sulfuric acid',
                          'parents': ['CHEBI:37826']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: cannot import name 'rdDecomposition' from "
               "'rdkit.Chem' "
               '(/Users/cjm/Library/Caches/pypoetry/virtualenvs/c3p-93U7KWO_-py3.11/lib/python3.11/site-packages/rdkit/Chem/__init__.py)',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 99,
    'num_false_positives': 100,
    'num_true_negatives': 11960,
    'num_false_negatives': 24,
    'num_negatives': None,
    'precision': 0.49748743718592964,
    'recall': 0.8048780487804879,
    'f1': 0.6149068322981367,
    'accuracy': 0.989821882951654}


"""
Classifies: CHEBI:25754 oxo carboxylic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_oxo_carboxylic_acid(smiles: str):
    """
    Determines if a molecule is an oxo carboxylic acid (contains both a carboxylic acid group
    and an aldehyde/ketone group).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an oxo carboxylic acid, False otherwise
        str: Reason for classification
    """
    # Create RDKit molecule object
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for carboxylic acid group
    carboxylic_acid_pattern = Chem.MolFromSmarts('[CX3](=O)[OX2H1]')
    has_carboxylic_acid = len(mol.GetSubstructMatches(carboxylic_acid_pattern)) > 0
    
    if not has_carboxylic_acid:
        return False, "No carboxylic acid group found"

    # Check for aldehyde group
    aldehyde_pattern = Chem.MolFromSmarts('[CX3H1](=O)[#6]')
    has_aldehyde = len(mol.GetSubstructMatches(aldehyde_pattern)) > 0

    # Check for ketone group 
    ketone_pattern = Chem.MolFromSmarts('[#6][CX3](=O)[#6]')
    has_ketone = len(mol.GetSubstructMatches(ketone_pattern)) > 0

    if not (has_aldehyde or has_ketone):
        return False, "No aldehyde or ketone group found"

    # Determine which oxo groups are present
    oxo_groups = []
    if has_aldehyde:
        oxo_groups.append("aldehyde")
    if has_ketone:
        oxo_groups.append("ketone")
        
    return True, f"Contains carboxylic acid and {' and '.join(oxo_groups)} groups"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25754',
                          'name': 'oxo carboxylic acid',
                          'definition': 'Any compound that has an aldehydic or '
                                        'ketonic group as well as a carboxylic '
                                        'acid group in the same molecule.',
                          'parents': ['CHEBI:33575']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 53,
    'num_false_positives': 100,
    'num_true_negatives': 4514,
    'num_false_negatives': 10,
    'num_negatives': None,
    'precision': 0.3464052287581699,
    'recall': 0.8412698412698413,
    'f1': 0.49074074074074076,
    'accuracy': 0.9764806499893094}


"""
Classifies: CHEBI:146181 mannotriose
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import BRICS
from rdkit.Chem.Draw import IPythonConsole

def is_mannotriose(smiles: str):
    """
    Determines if a molecule is a mannotriose (trisaccharide of 3 mannose units)
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a mannotriose, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check if molecule has expected formula for mannotriose (C18H32O16)
    formula = rdMolDescriptors.CalcMolFormula(mol)
    if formula != "C18H32O16":
        return False, f"Incorrect molecular formula: {formula}, expected C18H32O16"

    # Pattern matching for mannose unit
    mannose_pattern = Chem.MolFromSmarts("[CH2][CH]1O[CH]([CH]([CH]([CH]1O)O)O)O")
    if mannose_pattern is None:
        return None, "Error creating mannose pattern"
        
    # Find all mannose units
    matches = mol.GetSubstructMatches(mannose_pattern)
    
    if len(matches) != 3:
        return False, f"Found {len(matches)} mannose units, expected 3"
    
    # Check if units are connected by glycosidic bonds
    glycosidic_pattern = Chem.MolFromSmarts("[CH]1O[CH][CH][CH][CH][CH]1O[CH]")
    glycosidic_matches = mol.GetSubstructMatches(glycosidic_pattern)
    
    if len(glycosidic_matches) < 2:
        return False, "Mannose units not properly connected by glycosidic bonds"
        
    return True, "Contains 3 mannose units connected by glycosidic bonds"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:146181',
                          'name': 'mannotriose',
                          'definition': 'Any trisaccharide composed of 3 '
                                        'mannose moieties.',
                          'parents': ['CHEBI:25174', 'CHEBI:27150']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183911,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999891252929374}


"""
Classifies: CHEBI:33589 boranes
"""
from rdkit import Chem
from rdkit.Chem import rdMolDescriptors

def is_borane(smiles: str):
    """
    Determines if a molecule is a borane (molecular hydrides of boron).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a borane, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Get the number of boron and hydrogen atoms
    num_boron = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'B')
    num_hydrogen = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'H')

    # Check if it contains boron and hydrogen
    if num_boron == 0 or num_hydrogen == 0:
        return False, "Molecule does not contain boron and hydrogen"

    # Check if it contains other elements
    other_atoms = [atom for atom in mol.GetAtoms() if atom.GetSymbol() not in ['B', 'H']]
    if other_atoms:
        return False, "Molecule contains atoms other than boron and hydrogen"

    # Check if the hydrogen and boron atoms are connected
    hydrogen_neighbors = set()
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'H':
            for neighbor in atom.GetNeighbors():
                if neighbor.GetSymbol() == 'B':
                    hydrogen_neighbors.add(neighbor.GetIdx())

    # If all boron atoms have at least one hydrogen neighbor, it's a borane
    boron_atoms = [atom.GetIdx() for atom in mol.GetAtoms() if atom.GetSymbol() == 'B']
    if set(boron_atoms).issubset(hydrogen_neighbors):
        return True, "Molecule is a borane"
    else:
        return False, "Boron atoms are not connected to hydrogen atoms"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33589',
                          'name': 'boranes',
                          'definition': 'The molecular hydrides of boron.',
                          'parents': ['CHEBI:33588']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "name 'is_boranes' is not defined",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0,
    'f1': 0,
    'accuracy': None}


"""
Classifies: CHEBI:136637 steroid glucosiduronic acid anion
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import GetMorganFingerprintAsBitVect
from rdkit.Chem import rdMolDescriptors

def is_steroid_glucosiduronic_acid_anion(smiles: str):
    """
    Determines if a molecule is a steroid glucosiduronic acid anion.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a steroid glucosiduronic acid anion, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for presence of glucuronic acid anion moiety
    glucuronic_acid_pattern = Chem.MolFromSmarts("[OX2H1][CH]1[CH]([CH]([CH]([CH](O1)C(=O)[O-])[OH1])[OH1])[OH1]")
    if not mol.HasSubstructMatch(glucuronic_acid_pattern):
        return False, "No glucuronic acid anion moiety found"

    # Check for steroid core (four fused rings - three 6-membered, one 5-membered)
    steroid_core = Chem.MolFromSmarts("C1CC2CCC3C(C2)CCC4C3(CCC4)C1")
    if not mol.HasSubstructMatch(steroid_core):
        return False, "No steroid core structure found"

    # Count rings
    ring_info = mol.GetRingInfo()
    if len(ring_info.AtomRings()) < 5:  # At least 5 rings (4 for steroid core + 1 for glucuronic acid)
        return False, "Insufficient number of rings"

    # Check for negative charge
    total_charge = sum(atom.GetFormalCharge() for atom in mol.GetAtoms())
    if total_charge >= 0:
        return False, "No negative charge found"

    # Additional check for O-linkage between steroid and glucuronic acid
    o_linkage = Chem.MolFromSmarts("*-O-[CH]1O[CH][CH][CH][CH]1C(=O)[O-]")
    if not mol.HasSubstructMatch(o_linkage):
        return False, "No O-linkage between steroid and glucuronic acid found"

    # If all checks pass
    return True, "Molecule contains steroid core conjugated to glucuronic acid via O-linkage with negative charge"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:136637',
                          'name': 'steroid glucosiduronic acid anion',
                          'definition': 'A steroid conjugate anion formed by '
                                        'deprotonation of the carboxy group of '
                                        'any steroid glucosiduronic acid.',
                          'parents': ['CHEBI:50160', 'CHEBI:63551']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183873,
    'num_false_negatives': 6,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999673698464752}


"""
Classifies: CHEBI:138087 O-acyl-ADP-D-ribose(2-)
"""
from rdkit import Chem
from rdkit.Chem import rdMolDescriptors
from rdkit.Chem.rdqueries import AtomNumEqualsQueryAtom, BondBetweenAtomQuery, DescriptorQuery

def is_O_acyl_ADP_D_ribose_2__(smiles: str):
    """
    Determines if a molecule is an O-acyl-ADP-D-ribose(2-).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an O-acyl-ADP-D-ribose(2-), False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for two deprotonated phosphate groups
    deprotonated_phosphates = sum(atom.GetFormalCharge() == -1 and atom.GetSymbol() == 'O'
                                  for atom in mol.GetAtoms())
    if deprotonated_phosphates != 2:
        return False, "Does not contain two deprotonated phosphate groups"

    # Check for ADP-D-ribose substructure
    adp_ribose_query = Chem.MolFromSmarts('C1OC(COP(=O)(O)[O-])C(O)C1COP(=O)(O)[O-]')
    if not mol.HasSubstructMatch(adp_ribose_query):
        return False, "Does not contain ADP-D-ribose substructure"

    # Check for acyl group
    acyl_query = BondBetweenAtomQuery(DescriptorQuery('Atom.IsCarboxyl'), DescriptorQuery('Atom.IsOxygen'))
    if not mol.HasSubstructMatch(acyl_query):
        return False, "Does not contain an acyl group"

    return True, "Molecule is an O-acyl-ADP-D-ribose(2-)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:138087',
                          'name': 'O-acyl-ADP-D-ribose(2-)',
                          'definition': 'A nucleotide-sugar oxoanion obtained '
                                        'by deprotonation of the phosphate OH '
                                        'groups of any O-acyl-ADP-D-ribose; '
                                        'major species at pH 7.3.',
                          'parents': ['CHEBI:59737']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "cannot import name 'BondBetweenAtomQuery' from "
             "'rdkit.Chem.rdqueries' "
             '(/Users/cjm/Library/Caches/pypoetry/virtualenvs/c3p-93U7KWO_-py3.11/lib/python3.11/site-packages/rdkit/Chem/rdqueries.so)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:26995 thromboxane
"""
from rdkit import Chem
from rdkit.Chem import rdMolDescriptors

def is_thromboxane(smiles: str):
    """
    Determines if a molecule is a thromboxane, a class of oxygenated oxane derivatives that stimulate
    platelet aggregation and blood vessel constriction.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a thromboxane, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of an oxane ring
    ring_info = mol.GetRingInfo()
    oxane_ring = False
    for ring in ring_info.AtomRings():
        if len(ring) == 6:
            atoms = [mol.GetAtomWithIdx(idx) for idx in ring]
            oxane_atoms = [atom.GetSymbol() for atom in atoms]
            if set(oxane_atoms) == {'C', 'O'}:
                oxane_ring = True
                break

    if not oxane_ring:
        return False, "No oxane ring found"

    # Check for oxygenated substituents
    oxygenated = any(atom.GetSymbol() == 'O' for atom in mol.GetAtoms())
    if not oxygenated:
        return False, "No oxygenated substituents found"

    # Check for the presence of a carboxylate group
    carboxylate = any(atom.GetSymbol() == 'O' and atom.GetFormalCharge() == -1 for atom in mol.GetAtoms())
    if not carboxylate:
        return False, "No carboxylate group found"

    # If all conditions are met, classify as a thromboxane
    return True, "Molecule is a thromboxane"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26995',
                          'name': 'thromboxane',
                          'definition': 'A class of oxygenated oxane '
                                        'derivatives, originally derived from '
                                        'prostaglandin precursors in '
                                        'platelets, that stimulate aggregation '
                                        'of platelets and constriction of '
                                        'blood vessels.',
                          'parents': ['CHEBI:26347']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: module 'rdkit.Chem.AllChem' has no attribute "
               "'IsCyclic'",
    'attempt': 2,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 6683,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 0.5,
    'f1': 0.019417475728155338,
    'accuracy': 0.9851142225497421}


"""
Classifies: CHEBI:140377 (2S)-flavan-4-one
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is__2S__flavan_4_one(smiles: str):
    """
    Determines if a molecule is a (2S)-flavan-4-one.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a (2S)-flavan-4-one, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for basic flavan-4-one scaffold
    flavan_4_one_pattern = Chem.MolFromSmarts('[O;H0]=C1C[C@H](c2ccccc2)Oc3ccccc13')
    if not mol.HasSubstructMatch(flavan_4_one_pattern):
        return False, "Does not contain basic flavan-4-one scaffold"

    # Get matches for the core scaffold
    matches = mol.GetSubstructMatches(flavan_4_one_pattern)
    
    # Check stereochemistry at C-2 position
    for match in matches:
        c2_atom_idx = match[2]  # Index of C-2 carbon in the pattern
        c2_atom = mol.GetAtomWithIdx(c2_atom_idx)
        
        # Check if the chiral center has S configuration
        # In SMILES, @@ indicates R configuration, @ indicates S configuration
        if '@' in smiles and '@@' not in smiles:
            return True, "Contains (2S)-flavan-4-one scaffold with correct S stereochemistry"
            
    return False, "Does not have correct S stereochemistry at C-2 position"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:140377',
                          'name': '(2S)-flavan-4-one',
                          'definition': 'Any flavanone in which the chiral '
                                        'centre at position 2 has '
                                        'S-configuration.',
                          'parents': ['CHEBI:28863']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: cannot import name 'GetFormalCharge' from "
               "'rdkit.Chem.rdMolDescriptors' "
               '(/Users/cjm/Library/Caches/pypoetry/virtualenvs/c3p-93U7KWO_-py3.11/lib/python3.11/site-packages/rdkit/Chem/rdMolDescriptors.so)',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 49,
    'num_true_negatives': 183861,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.02,
    'recall': 0.5,
    'f1': 0.038461538461538464,
    'accuracy': 0.9997281308451869}


"""
Classifies: CHEBI:24631 hydrazines
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_hydrazines(smiles: str):
    """
    Determines if a molecule is a hydrazine or hydrazine derivative.
    Hydrazines contain an N-N single bond with possible substitutions.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a hydrazine, False otherwise
        str: Reason for classification
    """
    # Create RDKit molecule object
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Find N-N bonds
    n_n_bonds = []
    for bond in mol.GetBonds():
        atoms = [bond.GetBeginAtom(), bond.GetEndAtom()]
        if all(atom.GetAtomicNum() == 7 for atom in atoms):
            n_n_bonds.append(bond)

    if not n_n_bonds:
        return False, "No N-N bonds found"

    # Check each N-N bond
    for bond in n_n_bonds:
        # Get the nitrogen atoms
        n1, n2 = bond.GetBeginAtom(), bond.GetEndAtom()

        # Check if it's a single bond (hydrazine-like)
        if bond.GetBondType() == Chem.BondType.SINGLE:
            # Get substituents count
            subst_n1 = len([a for a in n1.GetNeighbors() if a.GetAtomicNum() != 7])
            subst_n2 = len([a for a in n2.GetNeighbors() if a.GetAtomicNum() != 7])
            
            total_subst = subst_n1 + subst_n2
            
            if total_subst == 0:
                return True, "Unsubstituted hydrazine"
            else:
                return True, f"Substituted hydrazine with {total_subst} substituent(s)"
                
        # Check for hydrazone-like structures (N-N=C)
        elif bond.GetBondType() == Chem.BondType.SINGLE or bond.GetBondType() == Chem.BondType.DOUBLE:
            for n_atom in [n1, n2]:
                for neighbor in n_atom.GetNeighbors():
                    if neighbor.GetAtomicNum() == 6:  # Carbon
                        bond_to_c = mol.GetBondBetweenAtoms(n_atom.GetIdx(), neighbor.GetIdx())
                        if bond_to_c.GetBondType() == Chem.BondType.DOUBLE:
                            return True, "Hydrazone derivative"

    return False, "Contains N-N bond but does not match hydrazine patterns"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24631',
                          'name': 'hydrazines',
                          'definition': 'Hydrazine (diazane) and its '
                                        'substituted derivatives.',
                          'parents': ['CHEBI:51143']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 6,
    'num_false_positives': 100,
    'num_true_negatives': 8620,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.05660377358490566,
    'recall': 1.0,
    'f1': 0.10714285714285715,
    'accuracy': 0.9885399954159981}